GSTR_APPFOLDER = 1
GSTR_CONFOLDER = 2
GSTR_LANFOLDER = 3
GSTR_TEMFOLDER = 4
GSTR_CE_FILE   = 5
GSTR_CONF_FILE = 6
GSTR_LOC_FILE  = 7
GSTR_SSET_FILE = 8
GSTR_LOCX_FILE  = 9
GSTR_CEX_FILE   = 10
GSTR_CONFX_FILE = 11
GSTR_TZ_FILE    = 12
GSTR_COUNTRY_FILE = 13
GSTR_TEXT_FILE = 14
GSTR_TIPS_FILE = 15
GSTR_HELP_FILE = 16



from gaurabda.GCGregorianDate import GCGregorianDate, Today
import gaurabda.GCEarthData as GCEarthData
import gaurabda.GCLocation as GCLocation
from gaurabda.GCMoonData import MOONDATA
import gaurabda.GCAyanamsha as GCAyanamsha
import gaurabda.GCStrings as GCStrings
import gaurabda.GCMath as GCMath
from gaurabda.GCTime import GCTime
from gaurabda.GCSunData import SUNDATA
import gaurabda.GCUT as GCUT

import math

def CalculateMidnightNaksatra(date,earth):
    moon = MOONDATA()
    date = GCGregorianDate(date = date)
    date.shour = 1.0
    jdate = date.GetJulianDetailed()
    moon.Calculate(jdate, earth)
    d = GCMath.putIn360( moon.longitude_deg - GCAyanamsha.GetAyanamsa(jdate))
    return math.floor(( d * 3.0) / 40.0 )


def GetNextNaksatra(ed, startDate, nextDate, forward=True):
    phi = 40.0/3.0
    jday = startDate.GetJulianComplete()
    moon = MOONDATA()
    d = GCGregorianDate(date = startDate)
    ayanamsa = GCAyanamsha.GetAyanamsa(jday)
    prev_naks = 0
    new_naks = -1
    dir = 1.0 if forward else -1.0
    scan_step = 0.5 * dir

    xd = GCGregorianDate()

    moon.Calculate(jday, ed)
    l1 = GCMath.putIn360(moon.longitude_deg - ayanamsa)
    prev_naks = int(math.floor(l1 / phi))

    counter = 0
    while counter < 20:
        xj = jday
        xd.Set(d)

        jday += scan_step
        d.shour += scan_step
        d.NormalizeHours()

        moon.Calculate(jday, ed)
        l2 = GCMath.putIn360(moon.longitude_deg - ayanamsa)
        new_naks = int(math.floor(l2/phi))
        if prev_naks != new_naks:
            jday = xj
            d.Set(xd)
            scan_step *= 0.5
            counter+=1
            continue
        else:
            l1 = l2
    nextDate.Set(d)
    return new_naks

def GetPrevNaksatra(ed, startDate, nextDate):
    return GetNextNaksatra(ed, startDate, nextDate, forward=False)

def writeXml(xml, loc, vc, nDaysCount):
    date = GCGregorianDate()

    xml.write("<xml>")
    xml.write("\t<request name=\"Naksatra\" version=\"")
    xml.write(GCStrings.getString(130))
    xml.write("\">\n")
    xml.write("\t\t<arg name=\"longitude\" val=\"")
    xml.write(str(loc.m_fLongitude))
    xml.write("\" />\n")
    xml.write("\t\t<arg name=\"latitude\" val=\"")
    xml.write(str(loc.m_fLatitude))
    xml.write("\" />\n")
    xml.write("\t\t<arg name=\"timezone\" val=\"")
    xml.write(str(loc.m_fTimezone))
    xml.write("\" />\n")
    xml.write("\t\t<arg name=\"startdate\" val=\"")
    xml.write(str(vc))
    xml.write("\" />\n")
    xml.write("\t\t<arg name=\"daycount\" val=\"")
    xml.write(str(nDaysCount))
    xml.write("\" />\n")
    xml.write("\t</request>\n")
    xml.write("\t<result name=\"Naksatra\">\n")

    d = GCGregorianDate(date = vc)
    d.tzone = loc.m_fTimezone
    dn = GCGregorianDate()
    dt = GCTime()
    sun = SUNDATA()
    earth = loc.GetEarthData()

    for i in range(30):
        nak = GetNextNaksatra(earth, d, dn)
        d.Set(dn)
        xml.write("\t\t<day date=\"")
        xml.write(str(d).strip())
        xml.write("\">\n")
        xml.write("\t\t\t<naksatra id=\"")
        xml.write(str(nak))
        xml.write("\" name=\"")
        xml.write(GCStrings.GetNaksatraName(nak))
        xml.write("\" ")
        dt.SetDegTime( d.shour * 360)
        xml.write("starttime=\"")
        xml.write(repr(dt))
        xml.write("\" />\n")

        sun.SunCalc(d, earth);
        xml.write("\t\t\t<sunrise time=\"");
        xml.write(repr(sun.rise))
        xml.write("\" />\n")

        xml.write("\t\t</day>\n");
        d.Set(dn)
        d.shour += 1.0/8.0


    xml.write("\t</result>\n")
    xml.write("</xml>\n")
    return 1

def GetEndHour(earth, yesterday, today):
    nend = GCGregorianDate()
    snd = GCGregorianDate(date = yesterday)
    snd.shour = 0.5
    GetNextNaksatra(earth, snd, nend)
    return nend.GetJulian() - today.GetJulian() + nend.shour

def unittests():
    GCUT.info('naksatra')
    e = GCEarthData.EARTHDATA()
    e.longitude_deg = 27.0
    e.latitude_deg = 45.0
    e.tzone = 1.0
    vc = Today()
    vc2 = GCGregorianDate()
    vc3 = GCGregorianDate(date = vc)
    vc3.NextDay()
    n = GetEndHour(e,vc,vc3)
    GCUT.msg('End hour: {}, {}'.format(n,repr(vc3)))
    import io
    s = io.StringIO()
    clr = GCLocation.GCLocation()
    writeXml(s, clr, vc, 10)
    GCUT.msg(s.getvalue())



from gaurabda.GCGregorianDate import GCGregorianDate,Today
from gaurabda.GCEarthData import EARTHDATA
from gaurabda.GCPancangaDate import GCGaurabdaDate
from gaurabda.GCLocation import GCLocation
from gaurabda.GCDayData import GCDayData,GetFirstDayOfYear
from gaurabda.GCEnums import MasaId
import gaurabda.GCTithi as GCTithi
import gaurabda.GCStrings as GCStrings
from math import floor
import gaurabda.GCUT as GCUT

def GetGaurabdaYear(vc, earth):
    day = GCDayData()
    day.DayCalc(vc, earth)
    day.MasaCalc(vc, earth)
    return day.nGaurabdaYear


def FormatDate(vc,va):
    return "{} {} {}\r\n{}, {} Paksa, {} Masa, {}".format(vc.day, GCStrings.GetMonthAbreviation(vc.month), vc.year, GCStrings.GetTithiName(va.tithi%15), GCStrings.GetPaksaName(va.paksa), GCStrings.GetMasaName(va.masa), va.gyear)

#===========================================================================
#
#===========================================================================

def Gaurabda2Gregorian(va, vc, earth):
    vc.Set(GCTithi.CalcTithiDate(va.gyear, va.masa, va.paksa, va.tithi % 15, earth))

#===========================================================================
#
#===========================================================================

def Gregorian2Gaurabda(vc, va, earth):
    day = GCDayData()
    day.DayCalc(vc, earth)
    va.masa = day.MasaCalc(vc, earth)
    va.tithi = day.nTithi
    va.gyear = day.nGaurabdaYear

def CalcEndDate(m_earth, vcStart, vaStart, vcEnd, vaEnd, nType, nCount):
    if nType==1:
        vcEnd.Set(vcStart)
        if nCount > 30240: nCount = 30240
        vcEnd.AddDays(nCount)
        Gregorian2Gaurabda(vcEnd, vaEnd, m_earth)
    elif nType==2:
        vcEnd.Set(vcStart)
        if nCount > 4320: nCount = 4320
        vcEnd.AddDays(nCount*7)
        Gregorian2Gaurabda(vcEnd, vaEnd, m_earth)
    elif nType==3:
        vcEnd.Set(vcStart)
        if nCount > 1080: nCount = 1080
        vcEnd.month += nCount
        while vcEnd.month > 12:
            vcEnd.year += 1
            vcEnd.month -= 12
        Gregorian2Gaurabda(vcEnd, vaEnd, m_earth)
    elif nType == 4:
        vcEnd.Set(vcStart)
        if nCount > 90: nCount = 90
        vcEnd.year += nCount
        Gregorian2Gaurabda(vcEnd, vaEnd, m_earth)
    elif nType == 5:
        vaEnd.Set(vaStart)
        if nCount > 30240: nCount = 30240
        vaEnd.tithi += nCount
        while vaEnd.tithi >= 30:
            vaEnd.tithi-=30
            vaEnd.masa+=1
        while vaEnd.masa >= 12:
            vaEnd.masa -= 12
            vaEnd.gyear+=1
        Gaurabda2Gregorian(vaEnd, vcEnd, m_earth)
    elif nType==6:
        vaEnd.Set(vaStart)
        if nCount > 1080: nCount = 1080
        vaEnd.masa = MasaToComboMasa(vaEnd.masa)
        if vaEnd.masa == MasaId.ADHIKA_MASA:
            vcEnd.Set(vcStart)
            vcEnd.month += nCount
            while vcEnd.month > 12:
                vcEnd.year+=1
                vcEnd.month -= 12
            Gregorian2Gaurabda(vcEnd, vaEnd, m_earth)
            vaEnd.tithi = vaStart.tithi
            Gaurabda2Gregorian(vaEnd, vcEnd, m_earth)
        else:
            vaEnd.masa += nCount
            while vaEnd.masa >= 12:
                vaEnd.masa -= 12
                vaEnd.gyear+=1
            vaEnd.masa = GCCalendar.ComboMasaToMasa(vaEnd.masa)
            Gaurabda2Gregorian(vaEnd, vcEnd, m_earth)
    elif nType==7:
        vaEnd.Set(vaStart)
        if nCount > 90: nCount = 90
        vaEnd.gyear += nCount
        Gaurabda2Gregorian(vaEnd, vcEnd, m_earth)
    return 1

def ComboMasaToMasa(nComboMasa):
    return 12 if nComboMasa == 12 else (nComboMasa + 11) % 12

def MasaToComboMasa(nMasa):
    return 12 if nMasa == 12 else (nMasa + 1) % 12

def writeFirstDayXml(xml,loc,vc):

    vcStart = GCGregorianDate(date=GetFirstDayOfYear(loc.GetEarthData(), vcStart.year))
    vcStart.InitWeekDay()

    # write
    xml.write("<xml>\n")
    xml.write("\t<request name=\"FirstDay\" version=\"")
    xml.write(GCStrings.getString(130))
    xml.write("\">\n")
    xml.write("\t\t<arg name=\"longitude\" val=\"")
    xml.write(str(loc.m_fLongitude))
    xml.write("\" />\n")
    xml.write("\t\t<arg name=\"latitude\" val=\"")
    xml.write(str(loc.m_fLatitude))
    xml.write("\" />\n")
    xml.write("\t\t<arg name=\"year\" val=\"")
    xml.write(str(vcStart.year))
    xml.write("\" />\n")
    xml.write("\t</request>\n")
    xml.write("\t<result name=\"FirstDay_of_GaurabdaYear\">\n")
    xml.write("\t\t<firstday date=\"")
    xml.write(str(vcStart))
    xml.write("\" dayweekid = \"")
    xml.write(str(vcStart.dayOfWeek))
    xml.write("\" dayweek=\"")
    xml.write(GCStrings.getString(vcStart.dayOfWeek))
    xml.write("\" />\n")
    xml.write("\t</result>\n")
    xml.write("</xml>\n")
    return 0


def unittests():
    GCUT.info('calendar')
    e = EARTHDATA()
    e.longitude_deg = 27.0
    e.latitude_deg = 45.0
    e.tzone = 1.0
    vc = Today()
    a = GetGaurabdaYear(vc,e)
    GCUT.msg('Gaurabda year:' + str(a))

    va = GCGaurabdaDate()
    Gregorian2Gaurabda(vc,va,e)
    GCUT.msg('Vatime: ' + str(va))

    va.prevMasa()
    va.tithi = 0
    vc2 = GCGregorianDate()
    Gaurabda2Gregorian(va,vc2,e)
    GCUT.msg('vctime2:' + str(vc2) + ' --> ' + str(va))



from .TCalendar import TCalendar
from .TAppDay import TAppDay
from .TCoreEvents import TCoreEvents
from .TMasaList import TMasaList
from .TToday import TToday
from .GCGregorianDate import GCGregorianDate
from .GCLocation import GCLocation



import os
import os.path
import json

import gaurabda.GCUT as GCUT

gcontinents = [
        "",
        "Europe", #1
        "Asia",   #2
        "Africa", #3
        "America",#4
        "Pacific",#5
        "Indiana",#6
        "Atlantic",#7
        ""
]

_modified = 0

gcountries = []

def InitWithFile(strFile):
    global gcountries
    if not os.path.exists(strFile):
        strFile = os.path.join(os.path.dirname(__file__), 'res', 'countries.json')
    with open(strFile,'rt',encoding='utf-8') as json_file:
        gcountries = json.load(json_file)
    return len(gcountries)

def FindCountry(name=None,abbr=None):
    if name!=None:
        for a in gcountries:
            if a['name']==name:
                return a
    elif abbr!=None:
        for a in gcountries:
            if a['abbr']==abbr:
                return a
    return None

def GetCountryName(w):
    bytes = int(w).to_bytes(2,byteorder='little',signed=False)
    abbr = chr(bytes[0]) + chr(bytes[1])
    for a in gcountries:
        if a['abbr'] == abbr:
            return a['name']
    return ""

def GetCountryContinentName(w):
    bytes = int(w).to_bytes(2,byteorder='little',signed=False)
    abbr = chr(bytes[0]) + chr(bytes[1])
    for a in gcountries:
        if a['abbr'] == abbr:
            return gcontinents[a['continent']]
    return ""

def GetCountryCount():
    return len(gcountries)

def GetCountryNameByIndex(nIndex):
    return gcountries[nIndex]['name']

def GetCountryContinentNameByIndex(nIndex):
    return gcontinents[gcountries[nIndex]['continent']]

def GetCountryAcronymByIndex(nIndex):
    return gcountries[nIndex]['abbr']

def SaveToFile(szFile):
    with open(szFile,'wt',encoding='utf-8') as json_file:
        json_file.write(json.dumps(gcountries,indent=4))

def AddCountry(abbr, name, continent):
    gcountries.append({
        'abbr': abbr,
        'name': name,
        'continent': continent
    })
    return len(gcountries)

def SetCountryName(index,name):
    if index>=0 and index<len(gcountries):
        gcountries[index]['name'] = name
    _modified = 1
    return 1

def GetCountryCode(index):
    return gcountries[index]['code']

def IsModified():
    return _modified

def unittests():
    GCUT.info('countries')
    GCUT.nval(InitWithFile('countries.json'),0,'load countries')
    GCUT.msg('count:' + str(len(gcountries)))
    country = FindCountry(name='India')
    GCUT.val(country['abbr'],'IN','find country')



from gaurabda.GCGregorianDate import GCGregorianDate,Today
import gaurabda.GCEarthData as GCEarthData
import gaurabda.GCMath as GCMath
import math
import gaurabda.GCUT as GCUT
import gaurabda.GCMoonData as GCMoonData
import gaurabda.GCSunData as GCSunData



#********************************************************************/
#                                                                   */
#  m1 - previous moon position                                      */
#  m2 - next moon position                                          */
#  s1 - previous sun position                                       */
#  s2 - next sun position                                           */
#                                                                   */
#  Test for conjunction of the sun and moon                         */
#  m1,s1 is in one time moment                                      */
#  m2,s2 is in second time moment                                   */
#                                                                   */
#  this function tests whether conjunction occurs between           */
#  these two moments                                                */
#                                                                   */
#********************************************************************/

def IsConjunction(m1, s1, s2, m2):
    if m2 < m1: m2 += 360.0
    if s2 < s1: s2 += 360.0
    if (m1 <= s1) and (s1 < s2) and (s2 <= m2):
        return True

    m1 = GCMath.putIn180(m1)
    m2 = GCMath.putIn180(m2)
    s1 = GCMath.putIn180(s1)
    s2 = GCMath.putIn180(s2)

    return (m1 <= s1) and (s1 < s2) and (s2 <= m2)

###################################/
# GET PREVIOUS CONJUNCTION OF THE SUN AND MOON
#
# THIS IS HELP FUNCTION FOR GetPrevConjunction(GCGregorianDate test_date,
#                                         GCGregorianDate &found, bool this_day, EARTHDATA earth)
#
# looking for previous sun-moon conjunction
# it starts from input day from 12:00 AM (noon) UTC
# so output can be the same day
# if output is the same day, then conjunction occurs between 00:00 AM and noon of this day
#
# date - input / output
# earth - input
# return value - sun longitude in degs
#
# error is when return value is greater than 999.0 deg

def GetPrevConjunction(date, earth, forward=False):
    prevSun = 0.0
    prevMoon = 0.0
    prevDiff = 0.0
    nowSun = 0.0
    nowMoon = 0.0
    nowDiff = 0.0
    dir = 1.0 if forward else -1.0
    moon = GCMoonData.MOONDATA()


    d = GCGregorianDate(date=date)
    d.shour = 0.5
    d.tzone = 0.0
    jd = d.GetJulian()

    # set initial data for input day
    # NOTE: for grenwich
    moon.Calculate(jd, earth)
    prevSun = GCSunData.GetSunLongitude(d)
    prevMoon = moon.longitude_deg
    prevDiff = GCMath.putIn180(prevSun - prevMoon)

    for bCont in range(32):
        if forward:
            d.NextDay()
        else:
            d.PreviousDay()
        jd += dir
        moon.Calculate(jd, earth)
        nowSun = GCSunData.GetSunLongitude(d)
        nowMoon = moon.longitude_deg
        nowDiff = GCMath.putIn180(nowSun - nowMoon)

        if IsConjunction(nowMoon, nowSun, prevSun, prevMoon):
            # now it calculates actual time and zodiac of conjunction
            if prevDiff == nowDiff: return 0
            x = math.fabs(nowDiff) / math.fabs(prevDiff - nowDiff)
            if x < 0.5:
                if forward: d.PreviousDay()
                d.shour = x + 0.5
            else:
                if not forward: d.NextDay()
                d.shour = x - 0.5
            date.Set(d)
            prevSun = GCMath.putIn360(prevSun)
            nowSun = GCMath.putIn360(nowSun)
            if math.fabs(prevSun - nowSun) > 10.0:
                return GCMath.putIn180(nowSun) + (GCMath.putIn180(prevSun) - GCMath.putIn180(nowSun))*x
            else:
                return nowSun + (prevSun - nowSun)*x
        prevSun = nowSun
        prevMoon = nowMoon
        prevDiff = nowDiff

    return 1000.0

###################################/
# GET NEXT CONJUNCTION OF THE SUN AND MOON
#
# Help function for GetNextConjunction(GCGregorianDate test_date, GCGregorianDate &found,
#                                      bool this_day, EARTHDATA earth)
#
# looking for next sun-moon conjunction
# it starts from input day from 12:00 AM (noon) UTC
# so output can be the same day
# if output is the same day, then conjunction occurs
# between noon and midnight of this day
#
# date - input / output
# earth - input
# return value - sun longitude
#
# error is when return value is greater than 999.0 deg

def GetNextConjunction(date, earth):
    return GetPrevConjunction(date, earth, forward=True)

#********************************************************************/
#                                                                   */
#                                                                   */
#                                                                   */
#                                                                   */
#                                                                   */
#********************************************************************/

def GetPrevConjunctionEx(test_date, found, this_conj, earth, forward = False):
    phi = 12.0
    dir = 1.0 if forward else -1.0
    if this_conj:
        test_date.shour += 0.2 * dir
        test_date.NormalizeHours()


    jday = test_date.GetJulianComplete()
    moon = GCMoonData.MOONDATA()
    d = GCGregorianDate(date = test_date)
    xd = GCGregorianDate()
    scan_step = 1.0
    prev_tit = 0
    new_tit = -1

    moon.Calculate(jday, earth)
    sunl = GCSunData.GetSunLongitude(d)
    l1 = GCMath.putIn180(moon.longitude_deg - sunl)
    prev_tit = int(math.floor(l1/phi))

    counter=0
    while counter<20:
        xj = jday
        xd.Set(d)

        jday += scan_step*dir
        d.shour += scan_step*dir
        d.NormalizeHours()

        moon.Calculate(jday, earth)
        sunl = GCSunData.GetSunLongitude(d)
        l2 = GCMath.putIn180(moon.longitude_deg - sunl)
        new_tit = int(math.floor(l2/phi))

        if forward:
            is_change = prev_tit < 0 and new_tit >= 0
        else:
            is_change = prev_tit >= 0 and new_tit < 0

        if is_change:
            jday = xj
            d.Set(xd)
            scan_step *= 0.5
            counter+=1
        else:
            l1 = l2
            prev_tit = new_tit

    found.Set(d)
    return sunl

#********************************************************************/
#                                                                   */
#                                                                   */
#                                                                   */
#                                                                   */
#                                                                   */
#********************************************************************/

def GetNextConjunctionEx(test_date, found, this_conj, earth):
    return GetPrevConjunctionEx(test_date, found, this_conj, earth, forward = True)


def unittests():
    GCUT.info('conjunctions')
    b = IsConjunction(10, 11, 12, 20)
    GCUT.val(b,True,'is conjunction')

    e = GCEarthData.EARTHDATA()
    e.longitude_deg = 27.0
    e.latitude_deg = 45.0
    e.tzone = 1.0
    vc = Today()
    vc2 = GCGregorianDate(date = vc)
    vc3 = GCGregorianDate(date = vc)

    l = GetPrevConjunction(vc2,e)
    GCUT.msg('Conjunction on: {}'.format(repr(vc2)))
    vc2.Set(vc)
    l = GetNextConjunctionEx(vc2,vc3,True,e)
    GCUT.msg('Conjunction on: {}'.format(repr(vc3)))



TITHI_KRSNA_PRATIPAT = 0
TITHI_KRSNA_DVITIYA = 1
TITHI_KRSNA_TRITIYA = 2
TITHI_KRSNA_CATURTI = 3
TITHI_KRSNA_PANCAMI = 4
TITHI_KRSNA_SASTI = 5
TITHI_KRSNA_SAPTAMI = 6
TITHI_KRSNA_ASTAMI = 7
TITHI_KRSNA_NAVAMI = 8
TITHI_KRSNA_DASAMI = 9
TITHI_KRSNA_EKADASI = 10
TITHI_KRSNA_DVADASI = 11
TITHI_KRSNA_TRAYODASI = 12
TITHI_KRSNA_CATURDASI = 13
TITHI_AMAVASYA = 14
TITHI_GAURA_PRATIPAT = 15
TITHI_GAURA_DVITIYA = 16
TITHI_GAURA_TRITIYA = 17
TITHI_GAURA_CATURTI = 18
TITHI_GAURA_PANCAMI = 19
TITHI_GAURA_SASTI = 20
TITHI_GAURA_SAPTAMI = 21
TITHI_GAURA_ASTAMI = 22
TITHI_GAURA_NAVAMI = 23
TITHI_GAURA_DASAMI = 24
TITHI_GAURA_EKADASI = 25
TITHI_GAURA_DVADASI = 26
TITHI_GAURA_TRAYODASI = 27
TITHI_GAURA_CATURDASI = 28
TITHI_PURNIMA = 29



from gaurabda.GCGregorianDate import GCGregorianDate,Today
from gaurabda.GCLocation import GCLocation
import gaurabda.GCMath as GCMath
import gaurabda.GCAyanamsha as GCAyanamsha
from gaurabda.GCTime import GCTime
from gaurabda.GCSunData import SUNDATA
import gaurabda.GCSunData as GCSunData
import gaurabda.GCStrings as GCStrings
from math import floor
import gaurabda.GCUT as GCUT

sankrantiDetermineType = 2

def GetSankrantiType():
    return sankrantiDetermineType

def SetSankrantiType(i):
    prev = sankrantiDetermineType
    sankrantiDetermineType = i
    return prev

def GetSankMethodName(i):
    snam = ["midnight to midnight",
        "sunrise to sunrise",
        "noon to noon",
        "sunset to sunset"]
    return snam[i]


#********************************************************************/
#  Finds next time when rasi is changed                             */
#                                                                   */
#  startDate - starting date and time, timezone member must be valid */
#  zodiac [out] - found zodiac sign into which is changed           */
#                                                                   */
#********************************************************************/
def GetNextSankranti(startDate):
    zodiac = 0
    step = 1.0
    count = 0
    prevday = GCGregorianDate()

    d = GCGregorianDate(date = startDate)

    prev = GCMath.putIn360( GCSunData.GetSunLongitude(d) - GCAyanamsha.GetAyanamsa(d.GetJulian()))
    prev_rasi = int(floor(prev / 30.0))

    while count < 20:
        prevday.Set(d)
        d.shour += step
        d.NormalizeHours()

        ld = GCMath.putIn360(GCSunData.GetSunLongitude(d) - GCAyanamsha.GetAyanamsa(d.GetJulian()))
        new_rasi = int(floor(ld/30.0))

        if prev_rasi != new_rasi:
            zodiac = new_rasi
            step *= 0.5
            d.Set(prevday)
            count+=1
            continue

    return d,zodiac


def writeXml(xml, loc, vcStart, vcEnd):
    dt = GCTime()
    zodiac = 0

    d = GCGregorianDate(date = vcStart)

    xml.write("<xml>\n")
    xml.write("\t<request name=\"Sankranti\" version=\"")
    xml.write(GCStrings.getString(130))
    xml.write("\">\n")
    xml.write("\t\t<arg name=\"longitude\" val=\"")
    xml.write(str(loc.m_fLongitude))
    xml.write("\" />\n")
    xml.write("\t\t<arg name=\"latitude\" val=\"")
    xml.write(str(loc.m_fLatitude))
    xml.write("\" />\n")
    xml.write("\t\t<arg name=\"timezone\" val=\"")
    xml.write(str(loc.m_fTimezone))
    xml.write("\" />\n")
    xml.write("\t\t<arg name=\"location\" val=\"")
    xml.write(loc.m_strName)
    xml.write("\" />\n")
    xml.write("\t\t<arg name=\"startdate\" val=\"")
    xml.write(repr(vcStart))
    xml.write("\" />\n")
    xml.write("\t\t<arg name=\"enddate\" val=\"")
    xml.write(repr(vcEnd))
    xml.write("\" />\n")
    xml.write("\t</request>\n")
    xml.write("\t<result name=\"SankrantiList\">\n")

    while d.IsBeforeThis(vcEnd):
        nextDate,zodiac = GetNextSankranti(d)
        d.Set(nextDate)
        d.InitWeekDay()
        xml.write("\t\t<sank date=\"")
        xml.write(str(d))
        xml.write("\" ")
        xml.write("dayweekid=\"")
        xml.write(str(d.dayOfWeek))
        xml.write("\" dayweek=\"")
        xml.write(GCStrings.getString(d.dayOfWeek))
        xml.write("\" ")

        dt.SetDegTime( 360 * d.shour )

        xml.write(" time=\"")
        xml.write(repr(dt))
        xml.write("\" >\n")
        xml.write("\t\t\t<zodiac sans=\"")
        xml.write(GCStrings.GetSankrantiName(zodiac))
        xml.write("\" eng=\"")
        xml.write(GCStrings.GetSankrantiNameEn(zodiac))
        xml.write("\" id=\"")
        xml.write(str(zodiac))
        xml.write("\" />\n")
        xml.write("\t\t</sank>\n")

        d.NextDay()
        d.NextDay()

    xml.write("\t</result>\n")
    xml.write("</xml>")

    return 1


def unittests():
    GCUT.info('sankranti')
    vc = Today()
    vc2 = GCGregorianDate()
    vc3 = GCGregorianDate(date = vc)
    vc3.AddDays(100)
    n = GetSankMethodName(GetSankrantiType())
    GCUT.msg('Sankranti Type: {}'.format(n))
    import io
    s = io.StringIO()
    clr = GCLocation()
    writeXml(s, clr, vc, vc3)
    GCUT.msg(s.getvalue())



import gaurabda.GCUT as GCUT
import gaurabda.GCStrings as GCStrings

class GCGaurabdaDate:
    def __init__(self,t=0,m=0,y=520):
        self.tithi = t
        self.masa = m
        self.gyear = y

    def __str__(self):
        return '{} {}, {} Masa, {}'.format(GCStrings.GetTithiName(self.tithi), GCStrings.GetPaksaName(self.paksa), GCStrings.GetMasaName(self.masa), self.gyear)

    @property
    def paksa(self):
        return 1 if self.tithi>=15 else 0

    def next(self):
        self.tithi+=1
        if self.tithi >= 30:
            self.tithi %= 30
            self.nextMasa()

    def nextMasa(self):
        self.masa+=1
        if self.masa >= 12:
            self.masa %= 12
            self.gyear+=1

    def prev(self):
        if self.tithi == 0:
            self.tithi = 29
            self.prevMasa()
        else:
            self.tithi-=1

    def prevMasa(self):
        if self.masa == 0:
            self.masa = 11
            self.gyear -= 1
        else:
            self.masa-=1

    def Set(self,va):
        self.tithi = va.tithi;
        self.masa = va.masa;
        self.gyear = va.gyear;

def unittests():
    GCUT.info('GCGaurabdaDate')
    va = GCGaurabdaDate()
    GCUT.val(va.tithi,0,'day')
    va.next()
    GCUT.val(va.tithi,1,'next')
    va.prev()
    va.prev()
    GCUT.val(va.tithi,29,'prev-tithi')
    GCUT.val(va.masa,11,'prev-masa')
    GCUT.val(va.gyear,519,'prev-gyear')



from gaurabda.GCGregorianDate import GCGregorianDate,Today
from gaurabda.GCLocation import GCLocation
from gaurabda.TCalendar import TCalendar
from gaurabda.GCCalendarDay import GCCalendarDay
from gaurabda.GCStringBuilder import GCStringBuilder,SBTF_RTF,SBTF_TEXT,SBTF_HTML

import gaurabda.GCTimeZone as GCTimeZone
import gaurabda.GCLayoutData as GCLayoutData
import gaurabda.GCStrings as GCStrings
import gaurabda.GCEarthData as GCEarthData
import gaurabda.GCDisplaySettings as GCDisplaySettings
import gaurabda.GCUT as GCUT
import datetime


class TToday:
    def __init__(self):
        self.currentDay = GCGregorianDate()
        self.calendar = TCalendar()

    def Calculate(self, dateTime, location):
        self.currentDay.Set(dateTime)
        self.currentDay.InitWeekDay()
        vc2 = GCGregorianDate(date = self.currentDay, addDays=-4)
        vc2.tzone = location.m_fTimezone
        self.calendar.CalculateCalendar(location, vc2, 9)

    def GetCurrentDay(self):
        i = self.calendar.FindDate(self.currentDay)
        return self.calendar.GetDay(i)

    def formatPlain(self, stream):
        p = self.GetCurrentDay()
        loc = self.calendar.m_Location
        vc = p.date

        sb = GCStringBuilder(stream)
        sb.Format = SBTF_TEXT

        if (p == None): return

        stream.write("{} ({}, {}, Timezone: {})\r\n\r\n[{} - {}]\r\n  {}, {} {}\r\n  {} {}, {} Gaurabda\r\n\r\n".format(loc.m_strName, GCEarthData.GetTextLatitude(loc.m_fLatitude), GCEarthData.GetTextLongitude(loc.m_fLongitude), GCTimeZone.GetTimeZoneName(loc.m_nTimezoneId), vc, GCStrings.getString(vc.dayOfWeek), GCStrings.GetTithiName(p.astrodata.nTithi), GCStrings.GetPaksaName(p.astrodata.nPaksa), GCStrings.getString(20), GCStrings.GetMasaName(p.astrodata.nMasa), GCStrings.getString(22), p.astrodata.nGaurabdaYear))

        self.WriteTodayInfo(sb,p)

    def formatRtf(self, stream):
        p = self.GetCurrentDay()
        loc = self.calendar.m_Location
        vc = p.date

        sb = GCStringBuilder(stream)
        sb.Format = SBTF_RTF
        sb.fontSizeH1 = GCLayoutData.textSizeH1
        sb.fontSizeH2 = GCLayoutData.textSizeH2
        sb.fontSizeText = GCLayoutData.textSizeText
        sb.fontSizeNote = GCLayoutData.textSizeNote

        sb.AppendDocumentHeader()

        stream.write("\\f2\\fs{} {} ".format(GCLayoutData.textSizeH1, vc.GetDateTextWithTodayExt()))
        stream.write("\\par\\f2\\fs{} {{\\fs{} {} }}\\line {} ({}, {}, Timezone: {})\\par\r\n\\par\r\n  {}, {} {}\\par\r\n  {} {}, {} Gaurabda\\par\r\n\\par\r\n".format( GCLayoutData.textSizeText, GCLayoutData.textSizeText+4, GCStrings.getString(p.date.dayOfWeek), loc.m_strName, GCEarthData.GetTextLatitude(loc.m_fLatitude), GCEarthData.GetTextLongitude(loc.m_fLongitude), GCTimeZone.GetTimeZoneName(loc.m_nTimezoneId), GCStrings.GetTithiName(p.astrodata.nTithi), GCStrings.GetPaksaName(p.astrodata.nPaksa), GCStrings.getString(20), GCStrings.GetMasaName(p.astrodata.nMasa), GCStrings.getString(22), p.astrodata.nGaurabdaYear))

        self.WriteTodayInfo(sb,p)


    def writeHtml(self, stream):
        p = self.GetCurrentDay()
        loc = self.calendar.m_Location
        vc = p.date
        sb = GCStringBuilder(stream)
        sb.Format = SBTF_HTML

        if (p == None): return

        stream.write("<html>\n<head>\n<title></title>")
        stream.write("<style>\n<!--\nbody {\n  font-family:Verdana;\n  font-size:9.5pt;\n}\n\ntd.hed {\n  font-size:9.5pt;\n  font-weight:bold;\n")
        stream.write("  background:#aaaaaa;\n  color:white;\n  text-align:center;\n  vertical-align:center;\n  padding-left:15pt;\n  padding-right:15pt;\n")
        stream.write("  padding-top:5pt;\n  padding-bottom:5pt;\n}\n-->\n</style>\n")
        stream.write("</head>\n")
        stream.write("<body>\n")
        stream.write("<h2>{}</h2>\n".format(vc.GetDateTextWithTodayExt()))
        stream.write("<h4>{}</h4>\n".format(loc.m_strFullName))
        stream.write("<p>  {}, {} {}<br>  {} {}, {} Gaurabda</p>".format(GCStrings.GetTithiName(p.astrodata.nTithi), GCStrings.GetPaksaName(p.astrodata.nPaksa), GCStrings.getString(20), GCStrings.GetMasaName(p.astrodata.nMasa), GCStrings.getString(22), p.astrodata.nGaurabdaYear))

        prevCountFest = 0

        stream.write("<p>")
        self.WriteTodayInfo(sb,p)

    def WriteTodayInfo(self,sb,p):
        sb.AppendLine()
        stream = sb.Target

        for ed in p.dayEvents:
            if 'disp' not in ed or ed['disp'] == -1 or GCDisplaySettings.getValue(ed['disp']):
                if 'spec' in ed:
                    sb.AppendHeader3(ed['text'],fillChar='-')
                else:
                    sb.AppendString(ed['text'])
                sb.AppendLine()

        if GCDisplaySettings.getValue(45):
            tda = p.astrodata.sun.rise
            sb.AppendLine()
            stream.write("Brahma Muhurta {} - {} ({})".format(tda.short(-96), tda.short(-48), GCStrings.GetDSTSignature(p.hasDST)))

        if GCDisplaySettings.getValue(29):
            sb.AppendLine()
            tda = p.astrodata.sun.rise
            stream.write("{} {} ".format(GCStrings.getString(51), tda.short() ))
            if (GCDisplaySettings.getValue(32)):
                stream.write(" sandhya {} - {} ", tda.short(-24), tda.short(24))
            stream.write(" ({})".format(GCStrings.GetDSTSignature(p.hasDST)))
            sb.AppendLine()


        if (GCDisplaySettings.getValue(30)):
            sb.AppendLine()
            tda = p.astrodata.sun.noon
            stream.write("{} {} ".format(GCStrings.getString(857), tda.short() ))
            if (GCDisplaySettings.getValue(32)):
                stream.write(" sandhya {} - {} ", tda.short(-24), tda.short(24))
            stream.write(" ({})".format(GCStrings.GetDSTSignature(p.hasDST)))
            sb.AppendLine()

        if (GCDisplaySettings.getValue(31)):
            sb.AppendLine()
            tda = p.astrodata.sun.set
            stream.write("{} {} ".format(GCStrings.getString(52), tda.short() ))
            if (GCDisplaySettings.getValue(32)):
                stream.write(" sandhya {} - {} ", tda.short(-24), tda.short(24))
            stream.write(" ({})".format(GCStrings.GetDSTSignature(p.hasDST)))
            sb.AppendLine()

        if (GCDisplaySettings.getValue(33)):
            sb.AppendLine()
            sb.AppendString(GCStrings.getString(51) + " info")
            sb.AppendLine()
            sb.AppendString("   Moon in {} {}".format(GCStrings.GetNaksatraName(p.astrodata.nNaksatra), GCStrings.getString(15)))

            if (GCDisplaySettings.getValue(47)):
                sb.AppendString(", {:.1f}% passed ({} Pada)".format(p.astrodata.nNaksatraElapse, GCStrings.getString(811+int(p.astrodata.nNaksatraElapse/25))))

            if (GCDisplaySettings.getValue(46)):
                sb.AppendString(", Moon in {} {}".format( GCStrings.GetSankrantiName(p.astrodata.nMoonRasi), GCStrings.getString(105)))

            sb.AppendString(", {} {}".format(GCStrings.GetYogaName(p.astrodata.nYoga), GCStrings.getString(104)))
            sb.AppendLine()
            sb.AppendString("   Sun in {} {}.".format(GCStrings.GetSankrantiName(p.astrodata.nSunRasi), GCStrings.getString(105)))
            sb.AppendLine()

        sb.AppendNote()
        sb.AppendDocumentTail()

def unittests():
    GCUT.info('today results')
    loc = GCLocation(data={
        'latitude': 48.150002,
        'longitude': 17.116667,
        'tzid': 321,
        'name': 'Bratislava, Slovakia'
    })
    earth = loc.GetEarthData()
    today = Today()

    tc = TToday()

    print('start calculate', datetime.datetime.now())
    tc.Calculate(today,loc)
    print('end calculate', datetime.datetime.now())

    with open('test/today.txt','wt') as wf:
        tc.formatPlain(wf)
    with open('test/today.rtf','wt') as wf:
        tc.formatRtf(wf)
    with open('test/today.html','wt') as wf:
        tc.writeHtml(wf)



import gaurabda.GCMath as GCMath
import gaurabda.GCTime as GCTime
import gaurabda.GCUT as GCUT
import gaurabda.GCAyanamsha as GCAyanamsha
from gaurabda.GCEnums import FastType,MahadvadasiType
import gaurabda.GCPancangaDate as GCPancangaDate
import gaurabda.GCDisplaySettings as GCDisplaySettings
import gaurabda.GCStrings as GCStrings
import gaurabda.GCCountry as GCCountry
import gaurabda.GCGregorianDate as GCGregorianDate
import gaurabda.GCTimeZone as GCTimeZone
import gaurabda.GCEarthData as GCEarthData
import gaurabda.GCStringBuilder as GCStringBuilder
import gaurabda.GCMoonData as GCMoonData
import gaurabda.GCSunData as GCSunData
import gaurabda.GCConjunction as GCConjunction
import gaurabda.GCNaksatra as GCNaksatra
import gaurabda.GCSankranti as GCSankranti
import gaurabda.GCDayData as GCDayData
import gaurabda.GCTithi as GCTithi
import gaurabda.GCYoga as GCYoga
import gaurabda.GCLocationList as GCLocationList
import gaurabda.GCEventList as GCEventList
import gaurabda.GCCalendar as GCCalendar
import TMasaList
import TCalendar
import TAppDay
import TCoreEvents
import TToday
import os

os.mkdir('test')

GCMath.unittests()
GCTime.unittests()
GCUT.info('enums')
GCUT.val(FastType.FAST_NULL,0, 'FastType')
GCUT.val(MahadvadasiType.EV_VIJAYA,0x110, 'MahadvadasiType')
GCAyanamsha.unittests()
GCPancangaDate.unittests()
GCDisplaySettings.unittests()
GCStrings.unittests()
GCCountry.unittests()
GCGregorianDate.unittests()
GCTimeZone.unittests()
GCEarthData.unittests()
GCStringBuilder.unittests()
GCMoonData.unittests()
GCSunData.unittests()
GCConjunction.unittests()
GCNaksatra.unittests()
GCSankranti.unittests()
GCTithi.unittests()
GCDayData.unittests()
GCYoga.unittests()
GCLocationList.unittests()
GCEventList.unittests()
GCCalendar.unittests()
# retype to if True: ... in case you need unittests for specific engine
if True: TMasaList.unittests()
if True: TCalendar.unittests()
if True: TAppDay.unittests()
if True: TCoreEvents.unittests()
TToday.unittests()



CCTYPE_DATE = 1
CCTYPE_S_ARUN = 10
CCTYPE_S_RISE = 11
CCTYPE_S_NOON = 12
CCTYPE_S_SET =  13
CCTYPE_TITHI =  20
CCTYPE_NAKS  =  21
CCTYPE_SANK  =  22
CCTYPE_CONJ  =  23
CCTYPE_YOGA  = 24
CCTYPE_KALA_START=  30
CCTYPE_KALA_END  =  31
CCTYPE_M_RISE = 41
CCTYPE_M_SET  = 42
CCTYPE_M_RASI = 45
CCTYPE_ASCENDENT = 50



import math

import gaurabda.GCEarthData as GCEarthData
import gaurabda.GCGregorianDate as GCGregorianDate
import gaurabda.GCTime as GCTime
import gaurabda.GCCoords as GCCoords
import gaurabda.GCUT as GCUT
import gaurabda.GCMath as GCMath
import gaurabda.GCAyanamsha as GCAyanamsha

class MOONDATA:
    def __init__(self):
        #   latitude from nodal (draconic) phase
        self.latitude_deg = 0.0
        #   longitude from sidereal motion
        self.longitude_deg = 0.0
        self.radius = 0.0
        self.rektaszension = 0.0
        self.declination = 0.0
        self.parallax = 0.0
        self.elevation = 0.0
        self.azimuth = 0.0

    def CalculateEcliptical(self,jdate):
        arg_lr = [
             [ 0, 0, 1, 0], [ 2, 0,-1, 0],     [ 2, 0, 0, 0],     [ 0, 0, 2, 0],
             [ 0, 1, 0, 0],     [ 0, 0, 0, 2],     [ 2, 0,-2, 0],     [ 2,-1,-1, 0],
             [ 2, 0, 1, 0],     [ 2,-1, 0, 0],     [ 0, 1,-1, 0],     [ 1, 0, 0, 0],
             [ 0, 1, 1, 0],     [ 2, 0, 0,-2],     [ 0, 0, 1, 2],     [ 0, 0, 1,-2],
             [ 4, 0,-1, 0],     [ 0, 0, 3, 0],     [ 4, 0,-2, 0],     [ 2, 1,-1, 0],
             [ 2, 1, 0, 0],     [ 1, 0,-1, 0],     [ 1, 1, 0, 0],     [ 2,-1, 1, 0],
             [ 2, 0, 2, 0],     [ 4, 0, 0, 0],     [ 2, 0,-3, 0],     [ 0, 1,-2, 0],
             [ 2, 0,-1, 2],     [ 2,-1,-2, 0],     [ 1, 0, 1, 0],     [ 2,-2, 0, 0],
             [ 0, 1, 2, 0],     [ 0, 2, 0, 0],     [ 2,-2,-1, 0],     [ 2, 0, 1,-2],
             [ 2, 0, 0, 2],     [ 4,-1,-1, 0],     [ 0, 0, 2, 2],     [ 3, 0,-1, 0],
             [ 2, 1, 1, 0],     [ 4,-1,-2, 0],     [ 0, 2,-1, 0],     [ 2, 2,-1, 0],
             [ 2, 1,-2, 0],     [ 2,-1, 0,-2],     [ 4, 0, 1, 0],     [ 0, 0, 4, 0],
             [ 4,-1, 0, 0],     [ 1, 0,-2, 0],     [ 2, 1, 0,-2],     [ 0, 0, 2,-2],
             [ 1, 1, 1, 0],     [ 3, 0,-2, 0],     [ 4, 0,-3, 0],     [ 2,-1, 2, 0],
             [ 0, 2, 1, 0],     [ 1, 1,-1, 0],     [ 2, 0, 3, 0],     [ 2, 0,-1,-2]]

        arg_b = [[ 0, 0, 0, 1],     [ 0, 0, 1, 1],     [ 0, 0, 1,-1],     [ 2, 0, 0,-1],
                [ 2, 0,-1, 1],     [ 2, 0,-1,-1],     [ 2, 0, 0, 1],     [ 0, 0, 2, 1],
                [ 2, 0, 1,-1],     [ 0, 0, 2,-1],  [ 2,-1, 0,-1],     [ 2, 0,-2,-1],
                [ 2, 0, 1, 1],     [ 2, 1, 0,-1],     [ 2,-1,-1, 1],     [ 2,-1, 0, 1],
                [ 2,-1,-1,-1],     [ 0, 1,-1,-1],     [ 4, 0,-1,-1],     [ 0, 1, 0, 1],
                [ 0, 0, 0, 3],     [ 0, 1,-1, 1],     [ 1, 0, 0, 1],     [ 0, 1, 1, 1],
                [ 0, 1, 1,-1],     [ 0, 1, 0,-1],     [ 1, 0, 0,-1],     [ 0, 0, 3, 1],
                [ 4, 0, 0,-1],     [ 4, 0,-1, 1],     [ 0, 0, 1,-3],     [ 4, 0,-2, 1],
                [ 2, 0, 0,-3],     [ 2, 0, 2,-1],     [ 2,-1, 1,-1],     [ 2, 0,-2, 1],
                [ 0, 0, 3,-1],     [ 2, 0, 2, 1],     [ 2, 0,-3,-1],     [ 2, 1,-1, 1],
                [ 2, 1, 0, 1],     [ 4, 0, 0, 1],     [ 2,-1, 1, 1],     [ 2,-2, 0,-1],
                [ 0, 0, 1, 3],     [ 2, 1, 1,-1],     [ 1, 1, 0,-1],     [ 1, 1, 0, 1],
                [ 0, 1,-2,-1],     [ 2, 1,-1,-1],     [ 1, 0, 1, 1],     [ 2,-1,-2,-1],
                [ 0, 1, 2, 1],     [ 4, 0,-2,-1],     [ 4,-1,-1,-1],     [ 1, 0, 1,-1],
                [ 4, 0, 1,-1],     [ 1, 0,-1,-1],     [ 4,-1, 0,-1],     [ 2,-2, 0, 1]]
        sigma_r = [-20905355, -3699111, -2955968,  -569925,
               48888,    -3149,   246158,  -152138,
             -170733,  -204586,  -129620,   108743,
              104755,    10321,        0,    79661,
              -34782,   -23210,   -21636,    24208,
               30824,    -8379,   -16675,   -12831,
              -10445,   -11650,    14403,    -7003,
                   0,    10056,     6322,    -9884,
                5751,        0,    -4950,     4130,
                   0,    -3958,        0,     3258,
                2616,    -1897,    -2117,     2354,
                   0,        0,    -1423,    -1117,
               -1571,    -1739,        0,    -4421,
                   0,        0,        0,        0,
                1165,        0,        0,     8752]

        sigma_l = [6288774, 1274027,  658314,  213618,
            -185116, -114332,   58793,   57066,
              53322,   45758,  -40923,  -34720,
             -30383,   15327,  -12528,   10980,
              10675,   10034,    8548,   -7888,
              -6766,   -5163,    4987,    4036,
               3994,    3861,    3665,   -2689,
              -2602,    2390,   -2348,    2236,
              -2120,   -2069,    2048,   -1773,
              -1595,    1215,   -1110,    -892,
               -810,     759,    -713,    -700,
                691,     596,     549,     537,
                520,    -487,    -399,    -381,
                351,    -340,     330,     327,
               -323,     299,     294,       0]

        sigma_b = [5128122,  280602,  277693,  173237,
              55413,   46271,   32573,   17198,
               9266,    8822,    8216,    4324,
               4200,   -3359,    2463,    2211,
               2065,   -1870,    1828,   -1794,
              -1749,   -1565,   -1491,   -1475,
              -1410,   -1344,   -1335,    1107,
               1021,     833,     777,     671,
                607,     596,     491,    -451,
                439,     422,     421,    -366,
               -351,     331,     315,     302,
               -283,    -229,     223,     223,
               -220,    -220,    -185,     181,
               -177,     176,     166,    -164,
                132,    -119,     115,     107]

        t = (jdate - 2451545.0)/36525.0

        #  (* mean elongation of the moon
        d = 297.8502042+(445267.1115168+(-0.0016300+(1.0/545868-1.0/113065000*t)*t)*t)*t

        #  (* mean anomaly of the sun
        m = 357.5291092+(35999.0502909+(-0.0001536+1.0/24490000*t)*t)*t

        #  (* mean anomaly of the moon
        ms = 134.9634114+(477198.8676313+(0.0089970+(1.0/69699-1.0/1471200*t)*t)*t)*t

        #  (* argument of the longitude of the moon
        f = 93.2720993+(483202.0175273+(-0.0034029+(-1.0/3526000+1.0/863310000*t)*t)*t)*t

        #  (* correction term due to excentricity of the earth orbit
        e = 1.0+(-0.002516-0.0000074*t)*t

        #  (* mean longitude of the moon
        ls = 218.3164591+(481267.88134236+(-0.0013268+(1.0/538841-1.0/65194000*t)*t)*t)*t

        #  (* arguments of correction terms
        a1 = 119.75+131.849*t
        a2 = 53.09+479264.290*t
        a3 = 313.45+481266.484*t

        sr = 0
        for i in range(60):
            temp =sigma_r[i]*GCMath.cosDeg( arg_lr[i][0]*d +arg_lr[i][1]*m +arg_lr[i][2]*ms +arg_lr[i][3]*f)
            if math.fabs(arg_lr[i][1])==1: temp = temp*e
            if math.fabs(arg_lr[i][1])==2: temp = temp*e*e
            sr =sr+temp

        sl = 0
        for i in range(60):
            temp =sigma_l[i]*GCMath.sinDeg( arg_lr[i][0]*d +arg_lr[i][1]*m +arg_lr[i][2]*ms +arg_lr[i][3]*f)
            if math.fabs(arg_lr[i][1])==1: temp =temp*e
            if math.fabs(arg_lr[i][1])==2: temp =temp*e*e
            sl =sl+temp

        #  (* correction terms
        sl =sl +3958*GCMath.sinDeg(a1) \
            +1962*GCMath.sinDeg(ls-f) \
            +318*GCMath.sinDeg(a2)
        sb = 0
        for i in range(60):
            temp = sigma_b[i]*GCMath.sinDeg( arg_b[i][0]*d + arg_b[i][1]*m + arg_b[i][2]*ms + arg_b[i][3]*f)
            if math.fabs(arg_b[i][1])==1: temp=temp*e
            if math.fabs(arg_b[i][1])==2: temp=temp*e*e
            sb=sb+temp

        #  (* correction terms
        sb =sb - 2235*GCMath.sinDeg(ls) + 382*GCMath.sinDeg(a3)  + 175*GCMath.sinDeg(a1-f) + 175*GCMath.sinDeg(a1+f)  +127*GCMath.sinDeg(ls-ms) - 115*GCMath.sinDeg(ls+ms)

        coords = GCCoords.GCEclipticalCoords()

        coords.longitude = ls+sl/1000000
        coords.latitude = sb/1000000
        coords.distance = 385000.56+sr/1000

        return coords

    def Calculate(self,jdate,earth):
        crd = self.CalculateEcliptical(jdate)
        eqc,crd = GCEarthData.eclipticalToEquatorialCoords(crd, jdate)

        self.radius = crd.distance
        self.longitude_deg =crd.longitude
        self.latitude_deg = crd.latitude

        self.rektaszension = eqc.rightAscension
        self.declination = eqc.declination

    def calc_horizontal(self,date, longitude, latitude):
        h = GCMath.putIn360(GCEarthData.star_time(date) - self.rektaszension+longitude)

        self.azimuth = GCMath.rad2deg( math.atan2(GCMath.sinDeg(h), GCMath.cosDeg(h)*GCMath.sinDeg(latitude) - GCMath.tanDeg(self.declination)*GCMath.cosDeg(latitude) ))

        self.elevation = GCMath.rad2deg( math.asin( GCMath.sinDeg(latitude) * GCMath.sinDeg(self.declination) + GCMath.cosDeg(latitude) * GCMath.cosDeg(self.declination) * GCMath.cosDeg(h)))

    def  correct_position(self, jdate, latitude, longitude, height):
        b_a=0.99664719

        u =GCMath.arcTanDeg(b_a*b_a*GCMath.tanDeg(latitude))
        rho_sin =b_a*GCMath.sinDeg(u)+height/6378140.0*GCMath.sinDeg(latitude)
        rho_cos =GCMath.cosDeg(u)+height/6378140.0*GCMath.cosDeg(latitude)

        self.parallax = GCMath.arcSinDeg(GCMath.sinDeg(8.794/3600)/(MoonDistance(jdate) / GCMath.AU))

        h = GCEarthData.star_time(jdate) - longitude - self.rektaszension;
        delta_alpha = GCMath.arcTanDeg( (-rho_cos*GCMath.sinDeg(self.parallax)*GCMath.sinDeg(h)) / (GCMath.cosDeg(self.declination) - rho_cos*GCMath.sinDeg(self.parallax)*GCMath.cosDeg(h)))
        self.rektaszension = self.rektaszension+delta_alpha
        self.declination =GCMath.arcTanDeg( (( GCMath.sinDeg(self.declination) -rho_sin*GCMath.sinDeg(self.parallax))*GCMath.cosDeg(delta_alpha))/ ( GCMath.cosDeg(self.declination) -rho_cos*GCMath.sinDeg(self.parallax)*GCMath.cosDeg(h)))

    def getTopocentricEquatorial(self, obs, jdate):
        b_a = 0.99664719
        tec = GCCoords.GCEquatorialCoords()

        altitude = 0

        #   geocentric position of observer on the earth surface
        #   10.1 - 10.3
        u = GCMath.arcTanDeg(b_a * b_a * GCMath.tanDeg(obs.latitude_deg))
        rho_sin = b_a * GCMath.sinDeg(u) + altitude / 6378140.0 * GCMath.sinDeg(obs.latitude_deg)
        rho_cos = GCMath.cosDeg(u) + altitude / 6378140.0 * GCMath.cosDeg(obs.latitude_deg)

        #   equatorial horizontal paralax
        #   39.1
        parallax = GCMath.arcSinDeg(GCMath.sinDeg(8.794 / 3600) / (self.radius / GCMath.AU))

        #   geocentric hour angle of the body
        h = GCEarthData.star_time(jdate) + obs.longitude_deg - self.rektaszension


        #   39.2
        delta_alpha = GCMath.arcTanDeg( (-rho_cos * GCMath.sinDeg(self.parallax) * GCMath.sinDeg(h)) / (GCMath.cosDeg(self.declination) - rho_cos * GCMath.sinDeg(self.parallax) * GCMath.cosDeg(h)))
        tec.rightAscension = self.rektaszension + delta_alpha
        tec.declination = GCMath.arcTanDeg( ((GCMath.sinDeg(self.declination) - rho_sin * GCMath.sinDeg(self.parallax)) * GCMath.cosDeg(delta_alpha)) / (GCMath.cosDeg(self.declination) - rho_cos * GCMath.sinDeg(self.parallax) * GCMath.cosDeg(h)))

        return tec



def MoonCalcElevation(e,vc):
    moon = MOONDATA()
    d = vc.GetJulianComplete()
    moon.Calculate(d, e)
    moon.correct_position(d, e.latitude_deg, e.longitude_deg, 0)
    moon.calc_horizontal(d, e.longitude_deg, e.latitude_deg)

    return moon.elevation

def CalcMoonTimes(e,vc,nDaylightSavingShift):
    rise = GCTime.GCTime()
    set = GCTime.GCTime()

    UT = 0.0
    i = 0
    prev_elev = 0.0
    nType = 0
    nFound = 0

    rise.SetValue(-1)
    set.SetValue(-1)

    #   inicializacia prvej hodnoty ELEVATION
    vc.shour = (-nDaylightSavingShift - 1.0) / 24.0
    prev_elev = MoonCalcElevation(e, vc)

    #   prechod cez vsetky hodiny
    UT = - 0.1 - nDaylightSavingShift
    while UT <= (24.1 - nDaylightSavingShift):
        vc.shour = UT / 24.0
        elev = MoonCalcElevation(e, vc)
        if prev_elev * elev <= 0.0:
            if prev_elev <= 0.0:
                nType = 0x1
            else:
                nType = 0x2

            a = UT - 1.0
            ae = prev_elev
            b = UT
            be = elev
            for i in range(20):
                c = (a + b) / 2.0
                vc.shour = c / 24.0
                ce = MoonCalcElevation(e, vc)
                if ae*ce <= 0.0:
                    be = ce
                    b = c
                else:
                    ae = ce
                    a = c

            if nType == 1:
                rise.SetDayTime((c + nDaylightSavingShift)/24.0)
            else:
                set.SetDayTime((c + nDaylightSavingShift)/24.0)
            nFound |= nType
            if nFound == 0x3: break
        prev_elev = elev
        UT+=1.0
    return rise,set

def GetNextMoonRasi(ed,startDate):
    nextDate = GCGregorianDate.GCGregorianDate(date=startDate)
    phi = 30.0
    jday = startDate.GetJulianComplete()
    moon = MOONDATA()
    d = GCGregorianDate.GCGregorianDate(date=startDate)
    ayanamsa = GCAyanamsha.GetAyanamsa(jday)
    scan_step = 0.5
    prev_naks = 0
    new_naks = -1

    xj = 0.0
    xd = GCGregorianDate.GCGregorianDate()

    moon.Calculate(jday, ed)
    l1 = GCMath.putIn360(moon.longitude_deg - ayanamsa)
    prev_naks = int(GCMath.Floor(l1 / phi))

    counter = 0
    while counter < 20:
        xj = jday
        xd.Set(d)

        jday += scan_step
        d.shour += scan_step
        if d.shour > 1.0:
            d.shour -= 1.0
            d.NextDay()

        moon.Calculate(jday, ed)
        l2 = GCMath.putIn360(moon.longitude_deg - ayanamsa);
        new_naks = int(GCMath.Floor(l2/phi))
        if prev_naks != new_naks:
            jday = xj
            d.Set(xd)
            scan_step *= 0.5
            counter+=1
            continue
        else:
            l1 = l2
    nextDate.Set(d)
    return new_naks,nextDate


def GetNextRise(e, vc, bRise):
    nFound = 0
    h = [0, 0, 0]
    hour = 1/24.0
    startHour = vc.shour

    track = GCGregorianDate.GCGregorianDate(date = vc)
    track.NormalizeValues()

    #   inicializacia prvej hodnoty ELEVATION
    h[0] = MoonCalcElevation(e, track)
    track.shour += hour
    h[1] = MoonCalcElevation(e, track)
    track.shour += hour
    h[2] = MoonCalcElevation(e, track)

    for c in range(24):
        has_change = False
        if bRise:
            has_change = h[1] < 0.0 and h[2] > 0.0
        else:
            has_change = h[1] > 0.0 and h[2] < 0.0
        if has_change:
            a = (h[2] - h[1]) / hour
            b = h[2] - a * track.shour
            track.shour = - b / a
            track.NormalizeValues()
            return track

        h[0] = h[1]
        h[1] = h[2]
        track.shour += hour
        h[2] = MoonCalcElevation(e, track)

    return track



def MoonDistance(jdate):
    arg_lr = [
         [ 0, 0, 1, 0], [ 2, 0,-1, 0], [ 2, 0, 0, 0], [ 0, 0, 2, 0], [ 0, 1, 0, 0],
         [ 0, 0, 0, 2], [ 2, 0,-2, 0], [ 2,-1,-1, 0], [ 2, 0, 1, 0], [ 2,-1, 0, 0],
         [ 0, 1,-1, 0], [ 1, 0, 0, 0], [ 0, 1, 1, 0], [ 2, 0, 0,-2], [ 0, 0, 1, 2],
         [ 0, 0, 1,-2], [ 4, 0,-1, 0], [ 0, 0, 3, 0], [ 4, 0,-2, 0], [ 2, 1,-1, 0],
         [ 2, 1, 0, 0], [ 1, 0,-1, 0], [ 1, 1, 0, 0], [ 2,-1, 1, 0], [ 2, 0, 2, 0],
         [ 4, 0, 0, 0], [ 2, 0,-3, 0], [ 0, 1,-2, 0], [ 2, 0,-1, 2], [ 2,-1,-2, 0],
         [ 1, 0, 1, 0], [ 2,-2, 0, 0], [ 0, 1, 2, 0], [ 0, 2, 0, 0], [ 2,-2,-1, 0],
         [ 2, 0, 1,-2], [ 2, 0, 0, 2], [ 4,-1,-1, 0], [ 0, 0, 2, 2], [ 3, 0,-1, 0],
         [ 2, 1, 1, 0], [ 4,-1,-2, 0], [ 0, 2,-1, 0], [ 2, 2,-1, 0], [ 2, 1,-2, 0],
         [ 2,-1, 0,-2], [ 4, 0, 1, 0], [ 0, 0, 4, 0], [ 4,-1, 0, 0], [ 1, 0,-2, 0],
         [ 2, 1, 0,-2], [ 0, 0, 2,-2], [ 1, 1, 1, 0], [ 3, 0,-2, 0], [ 4, 0,-3, 0],
         [ 2,-1, 2, 0], [ 0, 2, 1, 0], [ 1, 1,-1, 0], [ 2, 0, 3, 0], [ 2, 0,-1,-2]
       ]

    sigma_r = [
       -20905355, -3699111, -2955968,  -569925,    48888,    -3149,   246158,
         -152138,  -170733,  -204586,  -129620,   108743,   104755,    10321,
               0,    79661,   -34782,   -23210,   -21636,    24208,    30824,
           -8379,   -16675,   -12831,   -10445,   -11650,    14403,    -7003,
               0,    10056,     6322,    -9884,     5751,        0,    -4950,
            4130,        0,    -3958,        0,     3258,     2616,    -1897,
           -2117,     2354,        0,        0,    -1423,    -1117,    -1571,
           -1739,        0,    -4421,        0,        0,        0,        0,
            1165,        0,        0,     8752
      ]


    t = (jdate - 2451545.0)/36525.0

    #  (* mean elongation of the moon
    d = 297.8502042+(445267.1115168+(-0.0016300+(1.0/545868-1.0/113065000*t)*t)*t)*t

    #  (* mean anomaly of the sun
    m = 357.5291092+(35999.0502909+(-0.0001536+1.0/24490000*t)*t)*t

    #  (* mean anomaly of the moon
    ms = 134.9634114+(477198.8676313+(0.0089970+(1.0/69699-1.0/1471200*t)*t)*t)*t

    #  (* argument of the longitude of the moon
    f = 93.2720993+(483202.0175273+(-0.0034029+(-1.0/3526000+1.0/863310000*t)*t)*t)*t

    #  (* correction term due to excentricity of the earth orbit
    e = 1.0+(-0.002516-0.0000074*t)*t

    #  (* mean longitude of the moon
    ls = 218.3164591+(481267.88134236+(-0.0013268+(1.0/538841-1.0/65194000*t)*t)*t)*t

    sr = 0

    for i in range(60):
        temp =sigma_r[i]*GCMath.cosDeg( arg_lr[i][0]*d + arg_lr[i][1]*m + arg_lr[i][2]*ms + arg_lr[i][3]*f)
        if math.fabs(arg_lr[i][1])==1: temp = temp*e
        if math.fabs(arg_lr[i][1])==2: temp = temp*e*e
        sr =sr+temp


    return 385000.56+sr/1000


def unittests():
    GCUT.info('moon data')
    ed = GCEarthData.EARTHDATA()
    ed.longitude_deg = 20.0
    ed.latitude_deg = 45.0
    ed.tzone = 1.0
    vc = GCGregorianDate.GCGregorianDate(2000,1,1,0.5)
    GCUT.msg('Moon distance: ' + str(MoonDistance(vc.GetJulianComplete())))
    nd = GetNextRise(ed,vc,True)
    GCUT.msg('Next rise: ' + repr(nd))
    nd = GetNextRise(ed, vc, False)
    GCUT.msg('Next set: ' + repr(nd))
    a,b = GetNextMoonRasi(ed,vc)
    GCUT.msg('next moon rasi: {} {}'.format(a,b))
    a,b = CalcMoonTimes(ed,vc,1.0)
    GCUT.msg('moon times: {} {}'.format(repr(a),repr(b)))



SPEC_RETURNRATHA    =  3
SPEC_HERAPANCAMI    =  4
SPEC_GUNDICAMARJANA =  5
SPEC_GOVARDHANPUJA  =  6
SPEC_VAMANADVADASI  =  7
SPEC_VARAHADVADASI  =  8
SPEC_RAMANAVAMI     =  9
SPEC_JANMASTAMI     = 10
SPEC_RATHAYATRA     = 11
SPEC_GAURAPURNIMA   = 12
SPEC_NANDAUTSAVA    = 13
SPEC_MISRAFESTIVAL  = 14
SPEC_PRABHAPP       = 15



from gaurabda.GCEnums.CoreEventType import *
from gaurabda.GCEnums.KalaType import *
from gaurabda.GCEnums.GCDS import *
from gaurabda.GCGregorianDate import GCGregorianDate,Today
from gaurabda.GCLocation import GCLocation
import gaurabda.GCTimeZone as GCTimeZone
from gaurabda.GCSunData import SUNDATA,CalculateKala
from gaurabda.GCMoonData import MOONDATA,GetNextMoonRasi
from gaurabda.GCStringBuilder import GCStringBuilder,SBTF_RTF,SBTF_TEXT
import gaurabda.GCTithi as GCTithi
import gaurabda.GCNaksatra as GCNaksatra
import gaurabda.GCSankranti as GCSankranti
import gaurabda.GCConjunction as GCConjunction
import gaurabda.GCAyanamsha as GCAyanamsha
import gaurabda.GCYoga as GCYoga
import gaurabda.GCRasi as GCRasi
import gaurabda.GCStrings as GCStrings
import gaurabda.GCDisplaySettings as GCDisplaySettings
import gaurabda.GCLayoutData as GCLayoutData
from math import ceil,floor
import gaurabda.GCUT as GCUT
import datetime


class TDayEvent:
    def __init__(self):
        self.nType = 0
        self.nData = 0
        self.Time = GCGregorianDate()
        self.nDst = 0
        self.julianDay = 0

    def Set(self,de):
        nType = de.nType
        nData = de.nData
        Time.Set(de.Time)
        nDst = de.nDst

    def EventText(self):
        if self.nType == CCTYPE_S_ARUN:
            return 'arunodaya'
        elif self.nType == CCTYPE_S_RISE:
            return "sunrise"
        elif self.nType == CCTYPE_S_NOON:
            return "noon"
        elif self.nType == CCTYPE_S_SET:
            return "sunset"
        elif self.nType == CCTYPE_TITHI:
            return GCStrings.GetTithiName(self.nData) + " Tithi starts"
        elif self.nType == CCTYPE_NAKS:
            return GCStrings.GetNaksatraName(self.nData) + " Naksatra starts"
        elif self.nType == CCTYPE_YOGA:
            return GCStrings.GetYogaName(self.nData) + " Yoga starts"
        elif self.nType == CCTYPE_SANK:
            return "Sun enters {}".format(GCStrings.GetSankrantiName(self.nData))
        elif self.nType == CCTYPE_CONJ:
            return "conjunction in {} rasi".format(GCStrings.GetSankrantiName(self.nData))
        elif self.nType == CCTYPE_KALA_START:
            return GCStrings.GetKalaName(self.nData) + " starts"
        elif self.nType == CCTYPE_KALA_END:
            return GCStrings.GetKalaName(self.nData) + " ends"
        elif self.nType == CCTYPE_M_RISE:
            return "moonrise"
        elif self.nType == CCTYPE_M_SET:
            return "moonset"
        elif self.nType == CCTYPE_ASCENDENT:
            return GCStrings.GetSankrantiName(self.nData) + " ascendent"
        elif self.nType == CCTYPE_M_RASI:
            return "Moon enters {}".format(GCStrings.GetSankrantiName(self.nData))
        else:
            return ''

class TCoreEvents:
    header_text = {
        CCTYPE_DATE: " DATE ",
        CCTYPE_S_ARUN: " SUNRISE, SUNSET ",
        CCTYPE_S_RISE: " SUNRISE, SUNSET ",
        CCTYPE_S_NOON: " SUNRISE, SUNSET ",
        CCTYPE_S_SET: " SUNRISE, SUNSET ",
        CCTYPE_TITHI: " TITHI ",
        CCTYPE_NAKS: " NAKSATRA ",
        CCTYPE_SANK: " SANKRANTI ",
        CCTYPE_CONJ: " SUN-MOON CONJUNCTION ",
        CCTYPE_YOGA: " YOGA ",
        CCTYPE_KALA_START: " KALAS ",
        CCTYPE_KALA_END: " KALAS ",
        CCTYPE_M_RISE: " MOONRISE, MOONSET ",
        CCTYPE_M_SET: " MOONRISE, MOONSET ",
        CCTYPE_M_RASI: " MOON RASI ",
        CCTYPE_ASCENDENT: " ASCENDENT "
    }
    def __init__(self):
        self.m_vcStart = GCGregorianDate()
        self.m_vcEnd = GCGregorianDate()
        self.m_options = 0
        self.m_location = GCLocation()
        self.p_events = []
        self.b_sorted = True

    def AddEvent(self, inTime, inType, inData, inDst):
        p = TDayEvent()
        self.p_events.append(p)


        if inDst == 1:
            if (inTime.shour >= 2/24.0):
                inTime.shour += 1/24.0
                inTime.NormalizeValues()
                p.nDst = 1
        elif inDst == 2:
            inTime.shour += 1/24.0
            inTime.NormalizeValues()
            p.nDst = 1
        elif inDst == 3:
            if (inTime.shour <= 2/24.0):
                inTime.shour += 1/24.0
                inTime.NormalizeValues()
                p.nDst = 1
        p.Time.Set(inTime)
        p.Time.InitWeekDay()
        p.julianDay = p.Time.GetJulianComplete()
        p.nData = inData
        p.nType = inType
        return True

    def Sort(self):
        self.p_events = sorted(self.p_events, key = lambda k: k.julianDay)

    def CalculateEvents(self,loc,vcStart,vcEnd):
        sun = SUNDATA()
        ndst = 0

        self.m_location.Set(loc)
        self.m_vcStart.Set(vcStart)
        self.m_vcEnd.Set(vcEnd)

        vcNext = GCGregorianDate()
        earth = loc.GetEarthData()

        vc = GCGregorianDate(date = vcStart)
        vcAdd = GCGregorianDate(date = vcStart)
        vcAdd.InitWeekDay()

        previousLongitude = -100
        todayLongitude = 0
        fromTimeLimit = 0

        while vcAdd.IsBeforeThis(vcEnd):
            if (GCDisplaySettings.getValue(COREEVENTS_SUN)):
                ndst = GCTimeZone.determineDaylightChange(vcAdd, loc.m_nTimezoneId)
                sun.SunCalc(vcAdd, earth)

                vcAdd.shour = sun.arunodaya.GetDayTime()
                self.AddEvent(vcAdd, CCTYPE_S_ARUN, 0, ndst)

                vcAdd.shour = sunRise = sun.rise.GetDayTime()
                self.AddEvent(vcAdd, CCTYPE_S_RISE, 0, ndst)

                vcAdd.shour = sun.noon.GetDayTime()
                self.AddEvent(vcAdd, CCTYPE_S_NOON, 0, ndst)

                vcAdd.shour = sunSet = sun.set.GetDayTime()
                self.AddEvent(vcAdd, CCTYPE_S_SET, 0, ndst)
            else:
                ndst = GCTimeZone.determineDaylightChange(vcAdd, loc.m_nTimezoneId)
                sun.SunCalc(vcAdd, earth)
                sunRise = sun.rise.GetDayTime()
                sunSet = sun.set.GetDayTime()

            if (GCDisplaySettings.getValue(COREEVENTS_ASCENDENT)):
                todayLongitude = sun.longitude_deg
                vcAdd.shour = sunRise
                todaySunriseHour = sunRise
                if (previousLongitude < -10):
                    prevSunrise = GCGregorianDate(date = vcAdd)
                    prevSunrise.PreviousDay()
                    sun.SunCalc(prevSunrise, earth)
                    previousSunriseHour = sun.rise.GetDayTime() - 1
                    previousLongitude = sun.longitude_deg
                    fromTimeLimit = 0

                jd = vcAdd.GetJulianComplete()
                ayan = GCAyanamsha.GetAyanamsa(jd)
                r1 = GCMath.putIn360(previousLongitude - ayan) / 30
                r2 = GCMath.putIn360(todayLongitude - ayan) / 30

                while(r2 > r1 + 13):
                    r2 -= 12.0
                while(r2 < r1 + 11):
                    r2 += 12.0

                a = (r2 - r1) / (todaySunriseHour - previousSunriseHour)
                b = r2 - a * todaySunriseHour

                tr = ceil(r1)
                for tr in range(ceil(r1),ceil(r2)):
                    tm = ( tr - b ) / a
                    if (tm > fromTimeLimit):
                        vcNext.Set(vcAdd)
                        vcNext.shour = tm
                        vcNext.NormalizeValues()
                        self.AddEvent(vcNext, CCTYPE_ASCENDENT, tr, ndst)

                previousLongitude = todayLongitude
                previousSunriseHour = todaySunriseHour - 1
                fromTimeLimit = previousSunriseHour

            if (GCDisplaySettings.getValue(COREEVENTS_RAHUKALAM)):
                r1,r2 = CalculateKala(sunRise, sunSet, vcAdd.dayOfWeek,KT_RAHU_KALAM)
                vcAdd.shour = r1
                self.AddEvent(vcAdd, CCTYPE_KALA_START, KT_RAHU_KALAM, ndst)
                vcAdd.shour = r2
                self.AddEvent(vcAdd, CCTYPE_KALA_END, KT_RAHU_KALAM, ndst)

            if (GCDisplaySettings.getValue(COREEVENTS_YAMAGHANTI)):
                r1,r2 = CalculateKala(sunRise, sunSet, vcAdd.dayOfWeek, KT_YAMA_GHANTI)
                vcAdd.shour = r1
                self.AddEvent(vcAdd, CCTYPE_KALA_START, KT_YAMA_GHANTI, ndst)
                vcAdd.shour = r2
                self.AddEvent(vcAdd, CCTYPE_KALA_END, KT_YAMA_GHANTI, ndst)

            if (GCDisplaySettings.getValue(COREEVENTS_GULIKALAM)):
                r1,r2 = CalculateKala(sunRise, sunSet, vcAdd.dayOfWeek, KT_GULI_KALAM)
                vcAdd.shour = r1
                self.AddEvent(vcAdd, CCTYPE_KALA_START, KT_GULI_KALAM, ndst)
                vcAdd.shour = r2
                self.AddEvent(vcAdd, CCTYPE_KALA_END, KT_GULI_KALAM, ndst)

            if (GCDisplaySettings.getValue(COREEVENTS_ABHIJIT_MUHURTA)):
                r1,r2 = CalculateKala(sunRise, sunSet, vcAdd.dayOfWeek, KT_ABHIJIT)
                if (r1 > 0 and r2 > 0):
                    vcAdd.shour = r1
                    self.AddEvent(vcAdd, CCTYPE_KALA_START, KT_ABHIJIT, ndst)
                    vcAdd.shour = r2
                    self.AddEvent(vcAdd, CCTYPE_KALA_END, KT_ABHIJIT, ndst)

            vcAdd.NextDay()

        if (GCDisplaySettings.getValue(COREEVENTS_TITHI)):
            vcAdd.Set(vc)
            vcAdd.shour = 0.0
            while vcAdd.IsBeforeThis(vcEnd):
                nData = GCTithi.GetNextTithiStart(earth, vcAdd, vcNext)
                if (vcNext.GetDayInteger() < vcEnd.GetDayInteger()):
                    vcNext.InitWeekDay()
                    ndst = GCTimeZone.determineDaylightChange(vcNext, loc.m_nTimezoneId)
                    self.AddEvent(vcNext, CCTYPE_TITHI, nData, ndst)
                else:
                    break
                vcAdd.Set(vcNext)
                vcAdd.shour += 0.2
                if (vcAdd.shour >= 1.0):
                    vcAdd.shour -= 1.0
                    vcAdd.NextDay()

        if (GCDisplaySettings.getValue(COREEVENTS_NAKSATRA)):
            vcAdd.Set(vc)
            vcAdd.shour = 0.0
            while vcAdd.IsBeforeThis(vcEnd):
                nData = GCNaksatra.GetNextNaksatra(earth, vcAdd, vcNext)
                if (vcNext.GetDayInteger() < vcEnd.GetDayInteger()):
                    vcNext.InitWeekDay()
                    ndst = GCTimeZone.determineDaylightChange(vcNext, loc.m_nTimezoneId)
                    self.AddEvent(vcNext, CCTYPE_NAKS, nData, ndst)
                else:
                    break
                vcAdd.Set(vcNext)
                vcAdd.shour += 0.2
                if (vcAdd.shour >= 1.0):
                    vcAdd.shour -= 1.0
                    vcAdd.NextDay()

        if (GCDisplaySettings.getValue(COREEVENTS_YOGA)):
            vcAdd.Set(vc)
            vcAdd.shour = 0.0
            while vcAdd.IsBeforeThis(vcEnd):
                nData = GCYoga.GetNextYogaStart(earth, vcAdd, vcNext)
                if (vcNext.GetDayInteger() < vcEnd.GetDayInteger()):
                    vcNext.InitWeekDay()
                    ndst = GCTimeZone.determineDaylightChange(vcNext, loc.m_nTimezoneId)
                    self.AddEvent(vcNext, CCTYPE_YOGA, nData, ndst)
                else:
                    break
                vcAdd.Set(vcNext)
                vcAdd.shour += 0.2
                if (vcAdd.shour >= 1.0):
                    vcAdd.shour -= 1.0
                    vcAdd.NextDay()

        if (GCDisplaySettings.getValue(COREEVENTS_SANKRANTI)):
            vcAdd.Set(vc)
            vcAdd.shour = 0.0
            while vcAdd.IsBeforeThis(vcEnd):
                date,nData = GCSankranti.GetNextSankranti(vcAdd)
                vcNext.Set(date)
                if (vcNext.GetDayInteger() < vcEnd.GetDayInteger()):
                    vcNext.InitWeekDay()
                    ndst = GCTimeZone.determineDaylightChange(vcNext, loc.m_nTimezoneId)
                    self.AddEvent(vcNext, CCTYPE_SANK, nData, ndst)
                else:
                    break
                vcAdd.Set(vcNext)
                vcAdd.NextDay()

        if (GCDisplaySettings.getValue(COREEVENTS_MOONRASI)):
            vcAdd.Set(vc)
            vcAdd.shour = 0.0
            while vcAdd.IsBeforeThis(vcEnd):
                nData = GetNextMoonRasi(earth, vcAdd, vcNext)
                if (vcNext.GetDayInteger() < vcEnd.GetDayInteger()):
                    vcNext.InitWeekDay()
                    ndst = GCTimeZone.determineDaylightChange(vcNext, loc.m_nTimezoneId)
                    self.AddEvent(vcNext, CCTYPE_M_RASI, nData, ndst)
                else:
                    break
                vcAdd.Set(vcNext)
                vcAdd.shour += 0.5
                vcAdd.NormalizeValues()

        if (GCDisplaySettings.getValue(COREEVENTS_CONJUNCTION)):
            vcAdd.Set(vc)
            vcAdd.shour = 0.0
            while vcAdd.IsBeforeThis(vcEnd):
                dlong = GCConjunction.GetNextConjunctionEx(vcAdd, vcNext, True, earth)
                if (vcNext.GetDayInteger() < vcEnd.GetDayInteger()):
                    vcNext.InitWeekDay()
                    ndst = GCTimeZone.determineDaylightChange(vcNext, loc.m_nTimezoneId)
                    self.AddEvent(vcNext, CCTYPE_CONJ, GCRasi.GetRasi(dlong, GCAyanamsha.GetAyanamsa(vcNext.GetJulianComplete())), ndst)
                else:
                    break
                vcAdd.Set(vcNext)
                vcAdd.NextDay()

        if (GCDisplaySettings.getValue(COREEVENTS_MOON)):
            vcAdd.Set(vc)
            vcAdd.shour = 0.0
            while vcAdd.IsBeforeThis(vcEnd):
                vcNext.Set(MOONDATA.GetNextRise(earth, vcAdd, True))
                self.AddEvent(vcNext, CCTYPE_M_RISE, 0, ndst)

                vcNext.Set(MOONDATA.GetNextRise(earth, vcNext, False))
                self.AddEvent(vcNext, CCTYPE_M_SET, 0, ndst)

                vcNext.shour += 0.05
                vcNext.NormalizeValues()
                vcAdd.Set(vcNext)

        if self.b_sorted:
            self.Sort()

    def formatText(self,stream):
        sb = GCStringBuilder(stream)
        sb.Format = SBTF_TEXT

        stream.write("Events from {} to {}.\r\n\r\n".format( self.m_vcStart, self.m_vcEnd))
        stream.write("{}\r\n\r\n".format(self.m_location.m_strFullName))

        prevd = GCGregorianDate()
        prevd.day = 0
        prevd.month = 0
        prevd.year = 0
        prevt = -1

        last_header = ''
        for dnr in self.p_events:
            new_header = ''
            if self.b_sorted:
                new_header = " {} - {} ".format(dnr.Time, GCStrings.GetDayOfWeek(dnr.Time.dayOfWeek))
            else:
                new_header = header_text[dnr.nType]
            if last_header != new_header:
                sb.AppendLine()
                sb.AppendHeader3(new_header)
                sb.AppendLine()
                last_header = new_header

            stream.write("            {} {}    {}".format(dnr.Time.time_str(), GCStrings.GetDSTSignature(dnr.nDst), dnr.EventText()))
            sb.AppendLine()

        sb.AppendLine()
        sb.AppendNote()
        return 1

    def formatXml(self,strXml):
        strXml.write("<xml>\r\n<program version=\"{}\">\r\n<location longitude=\"{}\" latitude=\"{}\" timezone=\"{}\" dst=\"{}\" />\n".format(GCStrings.getString(130), self.m_location.m_fLongitude, self.m_location.m_fLatitude , self.m_location.m_fTimezone, GCTimeZone.GetTimeZoneName(self.m_location.m_nTimezoneId)))

        for dnr in self.p_events:
            strXml.write("  <event type=\"{}\" date=\"{}\" time=\"{}\" dst=\"{}\" />\n".format(dnr.EventText(), str(dnr.Time), dnr.Time.time_str(), dnr.nDst))

        strXml.write("</xml>\n")
        return 1

    def formatRtf(self,stream):
        sb = GCStringBuilder(stream)
        sb.Format = SBTF_RTF
        sb.fontSizeH1 = GCLayoutData.textSizeH1
        sb.fontSizeH2 = GCLayoutData.textSizeH2
        sb.fontSizeText = GCLayoutData.textSizeText
        sb.fontSizeNote = GCLayoutData.textSizeNote

        sb.AppendDocumentHeader()

        sb.AppendHeader1("Events")

        stream.write("\\par from {} to {}.\\par\r\n\\par\r\n".format( self.m_vcStart, self.m_vcEnd))
        stream.write("{}\\par\r\n\\par\r\n".format(self.m_location.m_strFullName))

        prevd = GCGregorianDate()
        prevd.day = 0
        prevd.month = 0
        prevd.year = 0
        prevt = -1

        last_header = ''
        for dnr in self.p_events:
            new_header = ''
            if self.b_sorted:
                new_header = " {} - {} ".format(dnr.Time, GCStrings.GetDayOfWeek(dnr.Time.dayOfWeek))
            else:
                new_header = header_text[dnr.nType]
            if last_header != new_header:
                sb.AppendLine()
                sb.AppendHeader2(new_header)
                sb.AppendLine()
                last_header = new_header

            stream.write("\\par            {} {}    {}".format(dnr.Time.time_str(), GCStrings.GetDSTSignature(dnr.nDst), dnr.EventText()))

        sb.AppendLine()
        sb.AppendNote()
        sb.AppendDocumentTail()
        return 1


    def writeHtml(self,stream):
        stream.write("<html>\n<head>\n<title>Core Events</title>\n\n")
        stream.write("<style>\n<!--\nbody {\n  font-family:Verdana;\n  font-size:11pt;\n}\n\ntd.hed {\n  font-size:11pt;\n  font-weight:bold;\n")
        stream.write("  background:#aaaaaa;\n  color:white;\n  text-align:center;\n  vertical-align:center;\n  padding-left:15pt;\n  padding-right:15pt;\n")
        stream.write("  padding-top:5pt;\n  padding-bottom:5pt;\n}\n-->\n</style>\n")
        stream.write("</head>\n")
        stream.write("<body>\n\n")
        stream.write("<h1 align=center>Events</h1>\n<p align=center>From {} to {}.</p>\n\n".format( self.m_vcStart, self.m_vcEnd))

        stream.write("<p align=center>{}</p>\n".format(self.m_location.m_strFullName))

        prevd = GCGregorianDate()
        prevd.day = 0
        prevd.month = 0
        prevd.year = 0
        prevt = -1

        stream.write("<table align=center><tr>\n")
        last_header = ''
        new_header = ''
        for dnr in self.p_events:
            if self.b_sorted:
                new_header = " {} - {} ".format(dnr.Time, GCStrings.GetDayOfWeek(dnr.Time.dayOfWeek))
            else:
                new_header = self.header_text[dnr.nType]
            if last_header != new_header:
                stream.write(f"<td class=\"hed\" colspan=2>{new_header}</td></tr>\n<tr>\n")
                last_header = new_header

            stream.write("<td>{}</td><td>{}</td></tr><tr>\n".format(dnr.EventText(), dnr.Time.time_str() ))

        stream.write("</tr></table>\n")
        stream.write("<hr align=center width=\"50%%\">\n<p align=center>Generated by {}</p>".format(GCStrings.getString(130)))
        stream.write("</body>\n</html>\n")
        return 1

def unittests():
    GCUT.info('core events results')
    loc = GCLocation(data={
        'latitude': 48.150002,
        'longitude': 17.116667,
        'tzid': 321,
        'name': 'Bratislava, Slovakia'
    })
    earth = loc.GetEarthData()
    today = Today()
    future = GCGregorianDate(date = today, addDays=100)

    tc = TCoreEvents()


    print('start calculate', datetime.datetime.now())
    tc.CalculateEvents(loc,today,future)
    print('end calculate', datetime.datetime.now())

    with open('test/events.xml','wt') as wf:
        tc.formatXml(wf)
    with open('test/events.txt','wt') as wf:
        tc.formatText(wf)
    with open('test/events.rtf','wt') as wf:
        tc.formatRtf(wf)
    with open('test/events.html','wt') as wf:
        tc.writeHtml(wf)



import json
import gaurabda.GCUT as GCUT

gss_default = [
    [0, 0, "ARTI", "Tithi at arunodaya"],#0
    [0, 0, "ARTM", "Arunodaya Time"],#1
    [0, 0, "SRTM", "Sunrise Time"],#2
    [0, 0, "SSTM", "Sunset Time"],#3
    [0, 0, "MRTM", "Moonrise Time"],#4
    [0, 0, "MSTM", "Moonset Time"],#5
    [1, 1, "FEST", "Festivals"],#6
    [0, 0, "KSAY", "Info about ksaya tithi"],#7
    [0, 0, "VRDH", "Info about vriddhi tithi"],#8
    [0, 0, "SLON", "Sun Longitude"],#9
    [0, 0, "MLON", "Moon Longitude"],#10
    [0, 0, "AYAN", "Ayanamsha value"],#11
    [0, 0, "JDAY", "Julian Day"],#12
    [1, 1, "CPUR", "Caturmasya Purnima System"], #13
    [0, 0, "CPRA", "Caturmasya Pratipat System"], #14
    [0, 0, "CEKA", "Caturmasya Ekadasi System"], #15
    [1, 1, "SANI", "Sankranti Info"], #16
    [1, 1, "EKAI", "Ekadasi Info"], #17
    [1, 1, "VHDR", "Masa Header Info"], #18
    [0, 0, "PHDR", "Month Header Info"], #19
    [0, 0, "EDNS", "Do not show empty days"], #20
    [0, 0, "SBEM", "Show begining of masa"], #21
    [1, 1, "F000", "Appearance days of the Lord"],#22
    [1, 1, "F001", "Events in the pastimes of the Lord"],#23
    [1, 1, "F002", "App, Disapp of Recent Acaryas"],#24
    [1, 1, "F003", "App, Disapp of Mahaprabhu's Associates and Other Acaryas"],#25
    [1, 1, "F004", "ISKCON's Historical Events"],#26
    [1, 1, "F005", "Bengal-specific Holidays"],#27
    [1, 1, "F006", "My Personal Events"], #28
    # BEGIN GCAL 1.4.3
    [1, 1, "TSSR", "Todat Sunrise"],  #29 Today sunrise
    [1, 1, "TSSN", "Today Noon"],  #30 today noon
    [1, 1, "TSSS", "Today Sunset"],  #31 today sunset
    [0, 0, "TSAN", "Sandhya Times"],  #32 today + sandhya times
    [1, 1, "TSIN", "Sunrise Info"],  #33 today sunrise info
    [0, 0, "ASIN", "Noon Time"],  #34 astro - noon time
    [1, 1, "NDST", "Notice about DST"], #35 notice about the change of the DST
    [1, 1, "DNAK", "Naksatra"], # 36 naksatra info for each day
    [1, 1, "DYOG", "Yoga"], #37 yoga info for each day
    [1, 1, "FFLG", "Fasting Flag"],#38
    [1, 1, "DPAK", "Paksa Info"],#39 paksa info
    [0, 0, "FDIW", "First Day in Week"],#40 first day in week
    [0, 0, "DRAS", "Rasi"], #41 moon rasi for each calendar day
    [0, 0, "OSFA", "Old Style Fasting text"], #42 old style fasting text
    [0, 0, "MLNT", "Name of month - type"], #43 month type name 0-vaisnava,1-bengal,2-hindu,3-vedic
    # END GCAL 1.4.3
    [0, 0, "EDBL", "Editable Default Events"], #44 editable default events
    [0, 0, "TSBM", "Today Brahma Muhurta"],     #45 brahma muhurta in today screen
    [0, 0, "TROM", "Today Rasi of the Moon"], # 46 rasi of the moon in today screen
    [0, 0, "TNPD", "Today Naksatra Pada details"], # 47 naksatra pada details in today screen
    [0, 0, "ADCS", "Child Names Suggestions"], # 48 child name suggestions in Appearance Day screen
    [0, 0, "MNFO", "Masa Name Format"], # 49 format of masa name
    [0, 0, "EPDR", "Ekadasi Parana details"], # 50 ekadasi parana details
    [0, 0, "ANIV", "Aniversary show format"], # 51 format of aniversary info
    [1, 1, "CE01", "Sun events"], # 52
    [1, 1, "CE02", "Tithi events"], #53
    [1, 1, "CE03", "Naksatra Events"], #54
    [1, 1, "CE04", "Sankranti Events"],#55
    [1, 1, "CE05", "Conjunction Events"],#56
    [0, 0, "CE06", "Rahu kalam"], #57
    [0, 0, "CE07", "Yama ghanti"], #58
    [0, 0, "CE08", "Guli kalam"], #59
    [0, 0, "CE09", "Moon events"], #60
    [0, 0, "CE10", "Moon rasi"], #61
    [0, 0, "CE11", "Ascendent"], #62
    [1, 1, "CE12", "Sort results core events"],#63
    [0, 0, "CE13", "Abhijit Muhurta"], #64
    [0, 0, "CE14", "Yoga Events"], #65
    [0, 0, "CE14", "Yoga Events"], #66
    [0, 0, "CE14", "Yoga Events"], #67
    [0, 0, "CE14", "Yoga Events"], #68
    [0, 0, "CE14", "Yoga Events"], #69
    [0, 0, "CE14", "Yoga Events"], #70
    [0, 0, "FKSP", "Festival on ksaya tithi: show on previous tithi"], #71
    [0, 0, None, None]
]

gss = gss_default

def getCount():
    return len(gss)

def getCountChanged():
    count = 0
    for a in gss:
        if a[0] != a[1]:
            count+=1
        a[1] = a[0]
    return count

def getSettingName(i):
    return gss[i][3]

def getValue(i):
    return gss[i][0]

def setValue(i,val):
    gss[i][0] = val
    gss[i][1] = val

def setValue(code,val):
    for a in gss:
        if a[2]==code:
            a[0] = val
            a[1] = val
            break

def readFile(fileName):
    default = True
    try:
        if os.path.exists(fileName):
            with open(fileName,'rt',encoding='utf-8') as json_file:
                gss = json.load(json_file)
                default = False
    except:
        default = True

    if default:
        gss = gss_default
    return default

def writeFile(fileName):
    try:
        with open(fileName,'wt',encoding='utf-8') as json_file:
            json_file.write(json.dumps(gss,indent=4))
    except:
        return False
    return True

def unittests():
    GCUT.info('display settings')
    GCUT.val(getValue(6),1,'get Value')



import math
import gaurabda.GCUT as GCUT

# pi
PI = 3.1415926535897932385

# 2 * pi
PI2 = 6.2831853071795864769

# pi / 180
RADS = 0.0174532925199432958

# distance of Earth from Sun in km
AU = 149597869.0

# astronomy data
EARTH_RADIUS = 6378.15
MOON_RADIUS = 1737.4
SUN_RADIUS = 695500

# julian day constants
J1999 = 2451180.0
J2000 = 2451545.0

# input value: arc in degrees
def cosDeg(x):
	return math.cos(x * RADS)

# input value: arc in degrees
def sinDeg(x):
	return math.sin(x * RADS)

def arccosDeg(x):
	return math.acos(x) / RADS

def Abs(d):
	return math.fabs(d)

# input value: arc in degrees
# it is calculating arctan(x/y)
# with testing values
def arcTan2Deg(x,y):
	return math.atan2(x, y) / RADS

# input value: arc in degrees
# output value: tangens
def tanDeg(x):
	return math.tan(x * RADS)

# input value: -1.0 .. 1.0
# output value: -180 deg .. 180 deg
def arcSinDeg(x):
	return math.asin(x) / RADS

def arcTanDeg(x):
	return math.atan(x) / RADS

# modulo 1
def putIn1(v):
	v2 = v - math.floor(v)
	while (v2 < 0.0):
		v2 += 1.0;
	while (v2 > 1.0):
		v2 -= 1.0
	return v2

# modulo 24
def putIn24(id):
	d = id
	while(d >= 24.0):
		d -= 24.0
	while(d < 0.0):
		d += 24.0
	return d

# modulo 360
def putIn360(id):
	d = id
	while(d >= 360.0):
		d -= 360.0
	while(d < 0.0):
		d += 360.0
	return d

# modulo 360 but in range -180deg .. 180deg
# used for comparision values around 0deg
# so difference between 359deg and 1 deg
# is not 358 degrees, but 2 degrees (because 359deg = -1 deg)
def putIn180(in_d):
	d = in_d
	while d < -180.0:
		d += 360.0
	while d > 180.0:
		d -= 360.0;
	return d

# sign of the number
# -1: number is less than zero
# 0: number is zero
# +1: number is greater than zero
def getSign(d):
	if d > 0.0:
		return 1
	if d < 0.0:
		return -1
	return 0

def deg2rad(x):
	return x * RADS

def rad2deg(x):
	return x / RADS

def getFraction(x):
	return x - math.floor(x)

def Max(a,b):
	if a > b:
		return a
	return b

def Min(a,b):
	if a < b:
		return a
	return b

def arcDistance(lon1,lat1,lon2,lat2):
    lat1 = PI / 2 - lat1
    lat2 = PI / 2 - lat2
    return arccosDeg(cosDeg(lat1) * cosDeg(lat2) + sinDeg(lat1) * sinDeg(lat2) * cosDeg(lon1 - lon2))

def arcDistanceDeg(lon1, lat1, lon2, lat2):
    return rad2deg(arcDistance(deg2rad(lon1), deg2rad(lat1), deg2rad(lon2), deg2rad(lat2)))

def Floor(d):
	return math.floor(d)

def unittests():
    GCUT.info('gc math')
    GCUT.val(Max(1,2), 2, 'max')
    GCUT.val(sinDeg(90), 1.0, 'sin')
    GCUT.val(cosDeg(0), 1.0, 'cos')
    GCUT.val(putIn24(30), 6, 'putIn24')
    GCUT.val(Floor(1.3),1.0, 'Floor')
    GCUT.val(Floor(-1.3),-2.0, 'Floor')



from gaurabda.GCMoonData import MOONDATA
from gaurabda.GCSunData import SUNDATA, GetSunLongitude
from gaurabda.GCGregorianDate import GCGregorianDate,Today
import gaurabda.GCEarthData as GCEarthData
import gaurabda.GCAyanamsha as GCAyanamsha
import gaurabda.GCMath as GCMath
import gaurabda.GCUT as GCUT
from math import floor

def GetNextYogaStart(ed, startDate, nextDate):
    phi = 40.0/3.0
    jday = startDate.GetJulianComplete()
    moon = MOONDATA()
    d = GCGregorianDate(date=startDate)
    xd = GCGregorianDate()
    scan_step = 0.5
    prev_tit = 0
    new_tit = -1
    ayanamsha = GCAyanamsha.GetAyanamsa(jday)
    moon.Calculate(jday, ed)
    sunl = GetSunLongitude(d)
    l1 = GCMath.putIn360( moon.longitude_deg + sunl - 2*ayanamsha)
    prev_tit = int(floor(l1/phi))

    counter = 0
    while counter < 20:
        xj = jday
        xd.Set(d)

        jday += scan_step
        d.shour += scan_step
        d.NormalizeHours()

        moon.Calculate(jday, ed)
        sunl = GetSunLongitude(d)
        l2 = GCMath.putIn360( moon.longitude_deg + sunl - 2*ayanamsha)
        new_tit = int(floor(l2/phi))

        if prev_tit != new_tit:
            jday = xj
            d.Set(xd)
            scan_step *= 0.5
            counter+=1
            continue
        else:
            l1 = l2
    nextDate.Set(d)

    return new_tit

def unittests():
    GCUT.info('yoga')
    e = GCEarthData.EARTHDATA()
    e.longitude_deg = 27.0
    e.latitude_deg = 45.0
    e.tzone = 1.0
    vc = Today()
    vc2 = GCGregorianDate()
    GetNextYogaStart(e,vc,vc2)
    print('Next yoga:', repr(vc2))



MESHA_SANKRANTI = 0
VRSABHA_SANKRANTI = 1
MITHUNA_SANKRANTI = 2
KATAKA_SANKRANTI = 3
SIMHA_SANKRANTI = 4
KANYA_SANKRANTI = 5
TULA_SANKRANTI = 6
VRSCIKA_SANKRANTI = 7
DHANUS_SANKRANTI = 8
MAKARA_SANKRANTI = 9
KUMBHA_SANKRANTI = 10
MINA_SANKRANTI = 11



from gaurabda.GCGregorianDate import GCGregorianDate,Today
from gaurabda.GCTime import GCTime
import gaurabda.GCLocation as GCLocation
import gaurabda.GCPancangaDate as GCPancangaDate
import gaurabda.GCEarthData as GCEarthData
import gaurabda.GCTithi as GCTithi
from gaurabda.GCPancangaDate import GCGaurabdaDate
from gaurabda.GCMoonData import MOONDATA
from gaurabda.GCSunData import GetSunLongitude, SUNDATA
from gaurabda.GCDayData import GCDayData, GetFirstDayOfYear, gGaurBeg
import gaurabda.GCMath as GCMath
import gaurabda.GCStrings as GCStrings
from math import floor
import gaurabda.GCUT as GCUT


def TITHI_EKADASI(a):
	return a in [10,25]

def TITHI_DVADASI(a):
	return a in [11,26]

def TITHI_TRAYODASI(a):
	return a in [12,27]

def TITHI_CATURDASI(a):
    return a in [13,28]

def TITHI_LESS_EKADASI(a):
    return a in [9,24,8,23]

def TITHI_LESS_DVADASI(a):
    return a in [10,25,9,24]

def TITHI_LESS_TRAYODASI(a):
    return a in [11,26,10,25]

def TITHI_FULLNEW_MOON(a):
    return a in [14,29]

def PREV_PREV_TITHI(a):
    return (a + 28) % 30

def PREV_TITHI(a):
    return (a + 29) % 30

def NEXT_TITHI(a):
    return (a + 1) % 30

def NEXT_NEXT_TITHI(a):
    return (a + 2) % 30

def TITHI_LESS_THAN(a,b):
    return a == PREV_TITHI(b) or a == PREV_PREV_TITHI(b)

def TITHI_GREAT_THAN(a,b):
    return a == NEXT_TITHI(b) or a == NEXT_NEXT_TITHI(b)

# TRUE when transit from A to B is between T and U
def TITHI_TRANSIT(t,u,a,b):
    return ((t==a) and (u==b)) or ((t==a) and (u==NEXT_TITHI(b))) or ((t==PREV_TITHI(a)) and (u == b))

#********************************************************************/
#                                                                   */
#   finds next time when starts next tithi                          */
#                                                                   */
#   timezone is not changed                                         */
#                                                                   */
#   return value: index of tithi 0..29                              */
#                 or -1 if failed                                   */
#********************************************************************/

def GetNextTithiStart(ed, startDate, nextDate, forward=True):
    phi = 12.0
    jday = startDate.GetJulianComplete()
    moon = MOONDATA()
    d = GCGregorianDate(date = startDate)
    xd = GCGregorianDate()
    dir = 1.0 if forward else -1.0
    scan_step = 0.5*dir
    prev_tit = 0
    new_tit = -1

    moon.Calculate(jday, ed)
    sunl = GetSunLongitude(d)
    l1 = GCMath.putIn360(moon.longitude_deg - sunl - 180.0)
    prev_tit = int(floor(l1/phi))

    counter = 0
    while counter < 20:
        xj = jday
        xd.Set(d)

        jday += scan_step
        d.shour += scan_step
        d.NormalizeHours()

        moon.Calculate(jday, ed)
        sunl = GetSunLongitude(d)
        l2 = GCMath.putIn360(moon.longitude_deg - sunl - 180.0)
        new_tit = int(floor(l2/phi))

        if prev_tit != new_tit:
            jday = xj
            d.Set(xd)
            scan_step *= 0.5
            counter+=1
            continue
        else:
            l1 = l2
    nextDate.Set(d)
    return new_tit


#********************************************************************/
#                                                                   */
#   finds previous time when starts next tithi                      */
#                                                                   */
#   timezone is not changed                                         */
#                                                                   */
#   return value: index of tithi 0..29                              */
#                 or -1 if failed                                   */
#********************************************************************/

def GetPrevTithiStart(ed, startDate, nextDate):
    return GetNextTithiStart(ed,startDate,nextDate,forward=False)


def GetTithiTimes(ed, vc, sunRise):
    d1 = GCGregorianDate()
    d2 = GCGregorianDate()

    vc.shour = sunRise
    GetPrevTithiStart(ed, vc, d1)
    GetNextTithiStart(ed, vc, d2)

    titBeg = d1.shour + d1.GetJulian() - vc.GetJulian()
    titEnd = d2.shour + d2.GetJulian() - vc.GetJulian()

    return titEnd - titBeg,titBeg,titEnd


#********************************************************************/
# Finds starting and ending time for given tithi                    */
#                                                                   */
# tithi is specified by Gaurabda year, masa, paksa and tithi number */
#      nGYear - 0..9999                                             */
#       nMasa - 0..12, 0-Madhusudana, 1-Trivikrama, 2-Vamana        */
#                      3-Sridhara, 4-Hrsikesa, 5-Padmanabha         */
#                      6-Damodara, 7-Kesava, 8-narayana, 9-Madhava  */
#                      10-Govinda, 11-Visnu, 12-PurusottamaAdhika   */
#       nPaksa -       0-Krsna, 1-Gaura                             */
#       nTithi - 0..14                                              */
#       earth  - used timezone                                      */
#                                                                   */
#********************************************************************/

def CalcTithiEnd(nGYear, nMasa, nPaksa, nTithi, earth, endTithi):
    d = GCGregorianDate(date = GetFirstDayOfYear(earth, nGYear + 1486))
    d.shour = 0.5
    d.tzone = earth.tzone

    return CalcTithiEndEx(d, nGYear, nMasa, nPaksa, nTithi, earth, endTithi)


def CalcTithiEndEx(vcStart, GYear, nMasa, nPaksa, nTithi, earth, endTithi):
    d = GCGregorianDate()
    dtemp = GCGregorianDate()
    day = GCDayData()
    tithi = 0
    counter = 0
    sunrise = 0.0
    start = GCGregorianDate()
    end = GCGregorianDate()
    start.shour = -1.0
    end.shour = -1.0
    start.day = start.month = start.year = -1
    end.day = end.month = end.year = -1

    d.Set(vcStart)

    i = 0
    while True:
        d.AddDays(13)
        day.DayCalc(d, earth)
        day.nMasa = day.MasaCalc(d, earth)
        gy = day.nGaurabdaYear
        i+=1
        if ((day.nPaksa == nPaksa) and (day.nMasa == nMasa)) or i > 30:
            break

    if i > 30:
        d.year = d.month = d.day = -1
        return d

    # we found masa and paksa
    # now we have to find tithi
    tithi = nTithi + nPaksa*15

    if day.nTithi == tithi:
        # loc1
        # find tithi juncts in this day and according to that times,
        # look in previous or next day for end and start of this tithi
        nType = 1
    else:
        if day.nTithi < tithi:
            # do increment of date until nTithi == tithi
            #   but if nTithi > tithi
            #       then do decrement of date
            counter = 0
            while counter < 30:
                d.NextDay()
                day.DayCalc(d, earth)
                if day.nTithi == tithi:
                    break
                if (day.nTithi < tithi ) and (day.nPaksa != nPaksa):
                    d.PreviousDay()
                    break
                if day.nTithi > tithi:
                    d.PreviousDay()
                    break
                counter+=1
            # somewhere is error
            if counter>=30:
                d.year = d.month = d.day = 0
                nType = 0
        else:
            # do decrement of date until nTithi <= tithi
            counter = 0
            while counter < 30:
                d.PreviousDay()
                day.DayCalc(d, earth)
                if day.nTithi == tithi:
                    break
                if (day.nTithi > tithi ) and (day.nPaksa != nPaksa):
                    break
                if day.nTithi < tithi:
                    break
                counter+=1
            # somewhere is error
            if counter >= 30:
                d.year = d.month = d.day = 0
                nType = 0

        if day.nTithi == tithi:
            # do the same as in loc1
            nType = 1
        else:
            # nTithi != tithi and nTithi < tithi
            # but on next day is nTithi > tithi
            # that means we will find start and the end of tithi
            # in this very day or on next day before sunrise
            nType = 2

    # now we know the type of day-accurancy
    # nType = 0 means, that we dont found a day
    # nType = 1 means, we find day, when tithi was present at sunrise
    # nType = 2 means, we found day, when tithi started after sunrise
    #                  but ended before next sunrise
    #
    sunrise = day.sun.sunrise_deg / 360 + earth.tzone/24

    if nType == 1:
        d1 = GCGregorianDate()
        d2 = GCGregorianDate()
        d.shour = sunrise
        GCTithi.GetPrevTithiStart(earth, d, d1)
        #d = d1
        #d.shour += 0.02
        GCTithi.GetNextTithiStart(earth, d, d2)

        endTithi.Set(d2)
        return d1
    elif nType == 2:
        d1 = GCGregorianDate()
        d2 = GCGregorianDate()
        d.shour = sunrise
        GCTithi.GetNextTithiStart(earth, d, d1)
        d.Set(d1)
        d.shour += 0.1
        d.NormalizeValues()
        GCTithi.GetNextTithiStart(earth, d, d2)

        endTithi.Set(d2)
        return d1

    # if nType == 0, then this algoritmus has some failure
    if nType == 0:
        d.year = 0
        d.month = 0
        d.day = 0
        d.shour = 0.0
        endTithi.Set(d)
    else:
        d.Set(start)
        endTithi.Set(end)

    return d

#********************************************************************/
#  Calculates Date of given Tithi                                   */
#********************************************************************/

def CalcTithiDate(nGYear, nMasa, nPaksa, nTithi, earth):
    i = 0
    gy = 0
    d = GCGregorianDate()
    dtemp = GCGregorianDate()
    day = GCDayData()
    tithi = 0
    counter = 0
    tmp = 0

    if nGYear >= 464 and nGYear < 572:
        tmp = gGaurBeg[(nGYear-464)*26 + nMasa*2 + nPaksa]
        d.month = (tmp & 0x3e0) >> 5
        d.day   = (tmp & 0x1f)
        d.year  = (tmp & 0xfffc00) >> 10
        d.tzone = earth.tzone
        d.NextDay()

        day.DayCalc(d, earth)
        day.nMasa = day.MasaCalc(d, earth)
        gy = day.nGaurabdaYear
    else:
        #d = GetFirstDayOfYear(earth, nGYear + 1486)
        d.day = 15
        d.month = 2 + nMasa
        d.year = nGYear + 1486
        if d.month > 12:
            d.month -= 12
            d.year+=1
        d.shour = 0.5
        d.tzone = earth.tzone

        i = 0
        while True:
            d.AddDays(13)
            day.DayCalc(d, earth)
            day.nMasa = day.MasaCalc(d, earth)
            gy = day.nGaurabdaYear
            i+=1
            if i>=30: break
            if (day.nPaksa == nPaksa) and (day.nMasa == nMasa): break

    if i >= 30:
        d.year = d.month = d.day = -1
        return d

    # we found masa and paksa
    # now we have to find tithi
    tithi = nTithi + nPaksa*15

    if day.nTithi == tithi:
        # loc1
        # find tithi juncts in this day and according to that times,
        # look in previous or next day for end and start of this tithi
        d.PreviousDay()
        day.DayCalc(d, earth)
        if (day.nTithi > tithi ) and (day.nPaksa != nPaksa):
            d.NextDay()
        return d

    if day.nTithi < tithi:
        # do increment of date until nTithi == tithi
        #   but if nTithi > tithi
        #       then do decrement of date
        counter = 0
        while counter < 16:
            d.NextDay()
            day.DayCalc(d, earth)
            if day.nTithi == tithi:
                return d
            if (day.nTithi < tithi ) and (day.nPaksa != nPaksa):
                return d
            if day.nTithi > tithi:
                return d
            counter+=1
        # somewhere is error
        d.year = d.month = d.day = 0
        return d
    else:
        # do decrement of date until nTithi <= tithi
        counter = 0
        while counter < 16:
            d.PreviousDay()
            day.DayCalc(d, earth)
            if day.nTithi == tithi:
                return d
            if (day.nTithi > tithi ) and (day.nPaksa != nPaksa):
                d.NextDay()
                return d
            if day.nTithi < tithi:
                d.NextDay()
                return d
            counter+=1
        # somewhere is error
        d.year = d.month = d.day = 0
        return d

    # now we know the type of day-accurancy
    # nType = 0 means, that we dont found a day
    # nType = 1 means, we find day, when tithi was present at sunrise
    # nType = 2 means, we found day, when tithi started after sunrise
    #                  but ended before next sunrise
    #

    return d

def writeXml(xml,loc,vc):
    date = GCGregorianDate()
    xml.write("<xml>\n")
    xml.write("\t<request name=\"Tithi\" version=\"")
    xml.write(GCStrings.getString(130))
    xml.write("\">\n")
    xml.write("\t\t<arg name=\"longitude\" val=\"")
    xml.write(str(loc.m_fLongitude))
    xml.write("\" />\n")
    xml.write("\t\t<arg name=\"latitude\" val=\"")
    xml.write(str(loc.m_fLatitude))
    xml.write("\" />\n")
    xml.write("\t\t<arg name=\"timezone\" val=\"")
    xml.write(str(loc.m_fTimezone))
    xml.write("\" />\n")
    xml.write("\t\t<arg name=\"startdate\" val=\"")
    xml.write(str(vc))
    xml.write("\" />\n")
    xml.write("\t</request>\n")
    xml.write("\t<result name=\"Tithi\">\n")

    d1 = GCGregorianDate()
    d2 = GCGregorianDate()
    dn = GCGregorianDate()
    dt = GCTime()
    earth = loc.GetEarthData()

    day = GCDayData()
    day.DayCalc(vc, earth)

    d = GCGregorianDate(date = vc)
    d.tzone = loc.m_fTimezone
    d.shour = day.sun.sunrise_deg/360.0 + loc.m_fTimezone/24.0

    GCTithi.GetPrevTithiStart(earth, d, d1)
    GCTithi.GetNextTithiStart(earth, d, d2)

    dt.SetDegTime( d1.shour * 360 )
    # start tithi at t[0]
    xml.write("\t\t<tithi\n\t\t\tid=\"")
    xml.write(str(day.nTithi))
    xml.write("\"\n")
    xml.write("\t\t\tname=\"")
    xml.write(GCStrings.GetTithiName(day.nTithi))
    xml.write("\"\n")
    xml.write("\t\t\tstartdate=\"")
    xml.write(str(d1))
    xml.write("\"\n")
    xml.write("\t\t\tstarttime=\"")
    xml.write(repr(dt))
    xml.write("\"\n")

    dt.SetDegTime( d2.shour * 360 )
    xml.write("\t\t\tenddate=\"")
    xml.write(str(d2))
    xml.write("\"\n")
    xml.write("\t\t\tendtime=\"")
    xml.write(repr(dt))
    xml.write("\"\n />")

    xml.write("\t</result>\n")
    xml.write("</xml>\n")

    return 1



def writeGaurabdaTithiXml(xml,loc,vaStart,vaEnd):
    gyearA = vaStart.gyear
    gyearB = vaEnd.gyear
    gmasa = vaStart.masa
    gpaksa = 1 if vaStart.tithi >= 15 else 0
    gtithi = vaStart.tithi % 15

    if gyearB < gyearA:
        gyearB = gyearA

    xml.write("<xml>\n")
    xml.write("\t<request name=\"Tithi\" version=\"")
    xml.write(GCStrings.getString(130))
    xml.write("\">\n")
    xml.write("\t\t<arg name=\"longitude\" val=\"")
    xml.write(str(loc.m_fLongitude))
    xml.write("\" />\n")
    xml.write("\t\t<arg name=\"latitude\" val=\"")
    xml.write(str(loc.m_fLatitude))
    xml.write("\" />\n")
    xml.write("\t\t<arg name=\"timezone\" val=\"")
    xml.write(str(loc.m_fTimezone))
    xml.write("\" />\n")
    if gyearA > 1500:
        xml.write("\t\t<arg name=\"year-start\" val=\"")
        xml.write(str(gyearA))
        xml.write("\" />\n")
        xml.write("\t\t<arg name=\"year-end\" val=\"")
        xml.write(str(gyearB))
        xml.write("\" />\n")
    else:
        xml.write("\t\t<arg name=\"gaurabdayear-start\" val=\"")
        xml.write(str(gyearA))
        xml.write("\" />\n")
        xml.write("\t\t<arg name=\"gaurabdayear-end\" val=\"")
        xml.write(str(gyearB))
        xml.write("\" />\n")
    xml.write("\t\t<arg name=\"masa\" val=\"")
    xml.write(str(gmasa))
    xml.write("\" />\n")
    xml.write("\t\t<arg name=\"paksa\" val=\"")
    xml.write(str(gpaksa))
    xml.write("\" />\n")
    xml.write("\t\t<arg name=\"tithi\" val=\"")
    xml.write(str(gtithi))
    xml.write("\" />\n")
    xml.write("\t</request>\n")
    xml.write("\t<result name=\"Tithi\">\n")


    earth = loc.GetEarthData()
    vcs = GCGregorianDate()
    vce = GCGregorianDate()
    today = GCGregorianDate()
    sun = SUNDATA()
    day = GCDayData()
    A = B = 0

    if gyearA > 1500:
        A = gyearA - 1487
        B = gyearB - 1485
    else:
        A = gyearA
        B = gyearB

    for A in range(A,B+1):
        vcs.Set(GCTithi.CalcTithiEnd(A, gmasa, gpaksa, gtithi, earth, vce))
        if gyearA > 1500:
            if (vcs.year < gyearA) or (vcs.year > gyearB):
                continue
        oTithi = gpaksa*15 + gtithi
        oMasa = gmasa
        oPaksa = gpaksa
        oYear = 0
        xml.write("\t<celebration\n")
        xml.write("\t\trtithi=\"")
        xml.write(GCStrings.GetTithiName(oTithi))
        xml.write("\"\n")
        xml.write("\t\trmasa=\"")
        xml.write(GCStrings.GetMasaName(oMasa))
        xml.write("\"\n")
        xml.write("\t\trpaksa=\"")
        xml.write('Gaura' if oPaksa else "Krsna")
        xml.write("\"\n")
        # test ci je ksaya
        today.Set(vcs)
        today.shour = 0.5
        sun.SunCalc(today, earth)
        sunrise = (sun.sunrise_deg + loc.m_fTimezone*15.0)/360
        if sunrise < vcs.shour:
            today.Set(vce)
            sun.SunCalc(today, earth)
            sunrise = (sun.sunrise_deg + loc.m_fTimezone*15.0)/360
            if sunrise < vce.shour:
                # normal type
                vcs.NextDay()
                xml.write("\t\ttype=\"normal\"\n")
            else:
                # ksaya
                vcs.NextDay()
                day.DayCalc(vcs, earth)
                oTithi = day.nTithi
                oPaksa = day.nPaksa
                oMasa = day.MasaCalc(vcs, earth)
                oYear = day.nGaurabdaYear
                xml.write("\t\ttype=\"ksaya\"\n")
        else:
            # normal, alebo prvy den vriddhi
            today.Set(vce)
            sun.SunCalc(today, earth)
            if (sun.sunrise_deg + loc.m_fTimezone*15.0)/360 < vce.shour:
                # first day of vriddhi type
                xml.write("\t\ttype=\"vriddhi\"\n")
            else:
                # normal
                xml.write("\t\ttype=\"normal\"\n")
        xml.write("\t\tdate=\"")
        xml.write(str(vcs))
        xml.write("\"\n")
        xml.write("\t\totithi=\"")
        xml.write(GCStrings.GetTithiName(oTithi))
        xml.write("\"\n")
        xml.write("\t\tomasa=\"")
        xml.write(GCStrings.GetMasaName(oMasa))
        xml.write("\"\n")
        xml.write("\t\topaksa=\"")
        xml.write('Gaura' if oPaksa else 'Krsna')
        xml.write("\"\n")
        xml.write("\t/>\n")

    xml.write("\t</result>\n")
    xml.write("</xml>\n")
    return 1

def writeGaurabdaNextTithiXml(xml,loc,vcStart,vaStart):
    gmasa = vaStart.masa
    gpaksa = 1 if vaStart.tithi >= 15 else 0
    gtithi = vaStart.tithi % 15

    xml.write("<xml>\n")
    xml.write("\t<request name=\"Tithi\" version=\"")
    xml.write(GCStrings.getString(130))
    xml.write("\">\n")
    xml.write("\t\t<arg name=\"longitude\" val=\"")
    xml.write(str(loc.m_fLongitude))
    xml.write("\" />\n")
    xml.write("\t\t<arg name=\"latitude\" val=\"")
    xml.write(str(loc.m_fLatitude))
    xml.write("\" />\n")
    xml.write("\t\t<arg name=\"timezone\" val=\"")
    xml.write(str(loc.m_fTimezone))
    xml.write("\" />\n")
    xml.write("\t\t<arg name=\"start date\" val=\"")
    xml.write(str(vcStart))
    xml.write("\" />\n")
    xml.write("\t\t<arg name=\"masa\" val=\"")
    xml.write(str(gmasa))
    xml.write("\" />\n")
    xml.write("\t\t<arg name=\"paksa\" val=\"")
    xml.write(str(gpaksa))
    xml.write("\" />\n")
    xml.write("\t\t<arg name=\"tithi\" val=\"")
    xml.write(str(gtithi))
    xml.write("\" />\n")
    xml.write("\t</request>\n")
    xml.write("\t<result name=\"Tithi\">\n")

    earth = loc.GetEarthData()
    vcs = GCGregorianDate()
    vce = GCGregorianDate()
    sun = SUNDATA()
    A = 0
    day = GCDayData()

    today = GCGregorianDate(date = vcStart)
    today.PreviousDay()
    vcStart.SubtractDays(15)
    for A in range(0,4):
        vcs.Set(GCTithi.CalcTithiEndEx(vcStart, 0, gmasa, gpaksa, gtithi, earth, vce))
        if not vcs.IsBeforeThis(today):
            oTithi = gpaksa*15 + gtithi
            oMasa = gmasa
            oPaksa = gpaksa
            oYear = 0
            xml.write("\t<celebration\n")
            xml.write("\t\trtithi=\"")
            xml.write(GCStrings.GetTithiName(oTithi))
            xml.write("\"\n")
            xml.write("\t\trmasa=\"")
            xml.write(GCStrings.GetMasaName(oMasa))
            xml.write("\"\n")
            xml.write("\t\trpaksa=\"")
            xml.write("Gaura" if oPaksa else "Krsna")
            xml.write("\"\n")
            # test ci je ksaya
            today.Set(vcs)
            today.shour = 0.5
            sun.SunCalc(today, earth)
            sunrise = (sun.sunrise_deg + loc.m_fTimezone*15.0)/360
            if sunrise < vcs.shour:
                today.Set(vce)
                sun.SunCalc(today, earth)
                sunrise = (sun.sunrise_deg + loc.m_fTimezone*15.0)/360
                if sunrise < vce.shour:
                    # normal type
                    vcs.NextDay()
                    xml.write("\t\ttype=\"normal\"\n")
                else:
                    # ksaya
                    vcs.NextDay()
                    day.DayCalc(vcs, earth)
                    oTithi = day.nTithi
                    oPaksa = day.nPaksa
                    oMasa = day.MasaCalc(vcs, earth)
                    oYear = day.nGaurabdaYear
                    xml.write("\t\ttype=\"ksaya\"\n")
            else:
                # normal, alebo prvy den vriddhi
                today.Set(vce)
                sun.SunCalc(today, earth)
                if (sun.sunrise_deg + loc.m_fTimezone*15.0)/360 < vce.shour:
                    # first day of vriddhi type
                    xml.write("\t\ttype=\"vriddhi\"\n")
                else:
                    # normal
                    xml.write("\t\ttype=\"normal\"\n")
            xml.write("\t\tdate=\"")
            xml.write(str(vcs))
            xml.write("\"\n")
            xml.write("\t\totithi=\"")
            xml.write(GCStrings.GetTithiName(oTithi))
            xml.write("\"\n")
            xml.write("\t\tomasa=\"")
            xml.write(GCStrings.GetMasaName(oMasa))
            xml.write("\"\n")
            xml.write("\t\topaksa=\"")
            xml.write("Gaura" if oPaksa else "Krsna")
            xml.write("\"\n")
            xml.write("\t/>\n")
            break
        else:
            vcStart.Set(vcs)
            vcs.NextDay()

    xml.write("\t</result>\n")
    xml.write("</xml>\n")
    return 1

# Finds first day of given masa and gaurabda year
def GetFirstDayOfMasa(earth,GYear,nMasa):
    return CalcTithiDate(GYear, nMasa, 0, 0, earth)


def unittests():
    GCUT.info('gc tithi')
    e = GCEarthData.EARTHDATA()
    e.longitude_deg = 27.0
    e.latitude_deg = 45.0
    e.tzone = 1.0
    vc = Today()
    vc2 = GCGregorianDate()
    vc3 = GCGregorianDate(date = vc)
    vc3.NextDay()

    if False:
        import io
        s = io.StringIO()
        clr = GCLocation.GCLocation()
        writeXml(s, clr, vc)
        GCUT.msg(s.getvalue())
        s.seek(0)
        writeGaurabdaTithiXml(s,clr,GCGaurabdaDate(2,1,512),GCGaurabdaDate(20,1,514))
        GCUT.msg('writeGaurabdaTithiXml')
        GCUT.msg(s.getvalue())

        s = io.StringIO()
        writeGaurabdaNextTithiXml(s,clr,vc,GCGaurabdaDate(2,1,512))
        GCUT.msg('writeGaurabdaNextTithiXml')
        GCUT.msg(s.getvalue())

    print(GetTithiTimes(e,vc,0.25))



import os
import os.path
import json
import math
import gaurabda.GCGregorianDate as GCGregorianDate
from gaurabda.GCGregorianDate import GCGregorianDate,GetMonthMaxDays
import gaurabda.GCUT as GCUT

gzone = []

def LoadFile(fileName):
    global gzone
    if not os.path.exists(fileName):
        fileName = os.path.join(os.path.dirname(__file__), 'res', 'timezones.json')
    with open(fileName,'rt',encoding='utf-8') as json_file:
        gzone = json.load(json_file)

def ID2INDEX(_id):
    for i,t in enumerate(gzone):
        if t['id'] == _id:
            return i
    return None

def GetTimeZone(id=None,index=None):
    if id!=None:
        index=ID2INDEX(id)
    if index==None:
        raise
    return gzone[index]

def GetTimeZoneName(id=None,index=None):
    if id!=None:
        index=ID2INDEX(id)
    if index==None:
        raise
    return gzone[index]['name']

def GetTimeZoneOffset(id=None,index=None):
    if id!=None:
        index=ID2INDEX(id)
    if index==None:
        raise
    return gzone[index]['offset']/60.0

def GetTimeZoneOffsetInteger(id=None,index=None):
    if id!=None:
        index=ID2INDEX(id)
    if index==None:
        raise
    return gzone[index]['offset']

def GetTimeZoneCount():
    return len(gzone)

def GetTimeZoneBias(id=None,index=None):
    if id!=None:
        index=ID2INDEX(id)
    if index==None:
        raise
    return gzone[index]['bias']

def GetDaylightTimeStartDate(nDst,nYear):
    nDst = ID2INDEX(nDst)
    a = gzone[nDst]

    vcStart = GCGregorianDate()
    vcStart.day = 1
    vcStart.month = a['startMonth']
    vcStart.year = nYear;
    if a['startType'] == 1:
        vcStart.day = a['startWeek']
    else:
        if a['startWeek'] == 5:
            vcStart.day = GCGregorianDate.GetMonthMaxDays(nYear, a['startMonth'])
            vcStart.InitWeekDay()
            while vcStart.dayOfWeek != a['startDay']:
                vcStart.PreviousDay()
        else:
            vcStart.day = 1
            vcStart.InitWeekDay()
            while vcStart.dayOfWeek != a['startDay']:
                vcStart.NextDay()
            vcStart.day += a['startWeek'] * 7
    vcStart.shour = 1/24.0
    return vcStart


def GetNormalTimeStartDate(nDst,nYear):
    vcStart = GCGregorianDate()
    vcStart.day = 1
    vcStart.month = 10
    vcStart.year = nYear
    vcStart.shour = 3/24.0
    return vcStart


def GetID(p):
    for a in gzone:
        if a['name']==p:
            return a['id']
    for a in gzone:
        if a['name'].startswith(p):
            return a['id']
    return int(p)

def GetTimeZoneOffsetText(d):
    sig = 1
    if d < 0.0:
        sig = -1
        d = -d
    else:
        sig = 1
    a4 = int(math.floor(d))
    a5 = int(math.floor((d - a4)*60 + 0.5))
    ss = '+' if sig>0 else '-'

    return '{}{:d}:{:02d}'.format(ss, a4, a5)

def GetTimeZoneOffsetTextArg(d):
    sig = 1
    if d < 0.0:
        sig = -1
        d = -d
    else:
        sig = 1
    a4 = int(math.floor(d))
    a5 = int(math.floor((d - a4)*60 + 0.5))
    ss = 'E' if sig>0 else 'W'

    return '{:d}{}{:02d}'.format(a4, ss, a5)


# n - is order number of given day
# x - is number of day in week (0-sunday, 1-monday, ..6-saturday)
# if x >= 5, then is calculated whether day is after last x-day

def is_n_xday(vc, n, x):
    xx = [1, 7, 6, 5, 4, 3, 2]

    # prvy den mesiaca
    fdm = xx[ (7 + vc.day - vc.dayOfWeek) % 7 ]

    # 1. x-day v mesiaci ma datum
    fxdm = xx[ (fdm - x + 7) % 7 ]

    # n-ty x-day ma datum
    if (n < 0) or (n >= 5):
        nxdm = fxdm + 28
        max = GetMonthMaxDays(vc.year, vc.month)
        while nxdm > max:
            nxdm -= 7
    else:
        nxdm = fxdm + (n - 1)* 7

    if vc.day >= nxdm: return 1
    return 0

# This table has 8 items for each line:
#  [0]: starting month
#  [4]: ending month
#
#  [1]: type of day, 0-day is given as n-th x-day of month, 1- day is given as DATE
#  [2]: for [1]==1 this means day of month
#     : for [1]==0 this order number of occurance of the given day (1,2,3,4 is acceptable, 5 means *last*)
#  [3]: used only for [1]==0, and this means day of week (0=sunday,1=monday,2=tuesday,3=wednesday,...)
#     : [1] to [3] are used for starting month
#  [5] to [7] is used for ending month in the same way as [1] to [3] for starting month
#
# EXAMPLE: (first line)   3 0 5 0 10 0 5 0
# [0] == 3, that means starting month is March
# [1] == 0, that means starting system is (day of week)
# [2] == 5, that would mean that we are looking for 5th occurance of given day in the month, but 5 here means,
#           that we are looking for *LAST* occurance of given day
# [3] == 0, this is *GIVEN DAY*, and it is SUNDAY
#
#         so, DST is starting on last sunday of March
#
# similarly analysed, DST is ending on last sunday of October
#

def GetDaylightBias(vc,tz):
    bias = 1
    if vc.month == tz['startMonth']:
        if tz['startType'] == 0:
            return is_n_xday(vc, tz['startWeek'], tz['startDay']) * bias
        else:
            return bias if vc.day >= tz['startWeek'] else 0
    elif vc.month == tz['endMonth']:
        if tz['endType'] == 0:
            return (1 - is_n_xday(vc, tz['endWeek'], tz['endDay']))*bias
        else:
            return 0 if vc.day >= tz['endWeek'] else bias
    else:
        if tz['startMonth'] > tz['endMonth']:
            # zaciatocny mesiac ma vyssie cislo nez koncovy
            # napr. pre australiu
            if (vc.month > tz['startMonth']) or (vc.month < tz['endMonth']):
                return bias
        else:
            # zaciatocny mesiac ma nizsie cislo nez koncovy
            # usa, europa, asia
            if (vc.month > tz['startMonth']) or (vc.month < tz['endMonth']):
                return bias

        return 0


def determineDaylightStatus(vc, id=None,index=None):
    if id != None:
        index = ID2INDEX(id)
    if index==None: raise
    return GetDaylightBias(vc, gzone[index]);

# return values
# 0 - DST is off, yesterday was off
# 1 - DST is on, yesterday was off
# 2 - DST is on, yesterday was on
# 3 - DST is off, yesterday was on
def determineDaylightChange(vc2,nIndex):
    t2 = determineDaylightStatus(vc2, nIndex)
    vc3 = GCGregorianDate()
    vc3.Set(vc2)
    vc3.PreviousDay()
    t1 = determineDaylightStatus(vc3, nIndex)
    if t1:
        if t2:
            return 2
        else:
            return 3
    elif t2:
        return 1
    else:
        return 0


def unittests():
    GCUT.info('timezones')
    LoadFile('timezones.json')
    GCUT.msg('Timezones:' + str(len(gzone)))
    tz = GetTimeZone(id=321)
    vc = GCGregorianDate(year=2020,month=4,day=4)
    GCUT.val(GetDaylightBias(vc,tz),1,'daylight bias')
    GCUT.val(GetTimeZoneOffsetText(1.0),'+1:00','offset text')



import gaurabda.GCEarthData as GCEarthData
import gaurabda.GCStrings as GCStrings
import gaurabda.GCTimeZone as GCTimeZone
import gaurabda.GCUT as GCUT
import re

def ParseDegreesFromString(str,bNorth):
    lat = lon = None
    m = re.match( r'(\+\-*)(\d+)([W|E|N|S])(\d+)', str )
    if m:
        val = int(m.group(2)) + int(m.group(4))/60
        if m.group(1)=='-':
            val = -val
        if m.group(3)=='W':
            lon = -val
        elif m.group(3)=='E':
            lon = val
        elif m.group(3)=='S':
            lat = -val
        elif m.group(3)=='N':
            lat = val
        elif m.group(3)=='.':
            if bNorth:
                lat = val
            else:
                lon = val
    return lon,lat

class GCLocation:
    def __init__(self,data=None):
        self.Empty()
        if data:
            if 'name' in data:
                self.m_strName = data['name']
            elif 'city' in data and 'country' in data:
                self.m_strCountry = data['country']
                self.m_strCity = data['city']
                self.m_strName = '{} ({})'.format(self.m_strCity, self.m_strCountry)
            else:
                self.m_strName = 'Noplace'
            self.m_fLongitude = data['longitude']
            self.m_fLatitude = data['latitude']
            if 'tzid' in data:
                self.m_nTimezoneId = data['tzid']
                self.m_strTimeZone = GCTimeZone.GetTimeZoneName(id=self.m_nTimezoneId)
                self.m_fTimezone = GCTimeZone.GetTimeZoneOffset(id=self.m_nTimezoneId)
            else:
                if 'tzname' in data:
                    self.m_strTimeZone = data['tzname']
                if 'offset' in data:
                    self.m_fTimezone = data['tzoffset']

    @property
    def m_strLongitude(self):
        return GCEarthData.GetTextLongitude(self.m_fLongitude)

    @property
    def m_strLatitude(self):
        return GCEarthData.GetTextLatitude(self.m_fLatitude)

    @property
    def m_strFullName(self):
        return str(self)


    def data(self):
        return {
            'city': self.m_strCity,
            'country': self.m_strCountry,
            'name': self.m_strName,
            'latitude': self.m_fLatitude,
            'longitude': self.m_fLongitude,
            'offset': self.m_fTimezone,
            'tzid': self.m_nTimezoneId,
            'tzname': self.m_strTimeZone
        }

    def Empty(self):
        self.m_strTimeZone = ''
        self.m_strName = ''
        self.m_fLatitude = 0.0
        self.m_fLongitude = 0.0
        self.m_fTimezone = 0.0
        self.m_nTimezoneId = 0
        self.m_strCountry = ''
        self.m_strCity = ''

    def __str__(self):
        return "{} ({}, {}, {}: {})".format(self.m_strName, self.m_strLatitude, self.m_strLongitude, GCStrings.getString(12), self.m_strTimeZone)

    def __dict__(self):
        return self.data()

    def copy(self):
        return GCLocation(data = self.data())

    def GetEarthData(self):
        e = GCEarthData.EARTHDATA()
        e.dst = self.m_nTimezoneId
        e.latitude_deg = self.m_fLatitude
        e.longitude_deg = self.m_fLongitude
        e.tzone = self.m_fTimezone
        return e

    def Set(self,L):
        if isinstance(L,GCEarthData.EARTHDATA):
            self.m_fLongitude = L.longitude_deg
            self.m_fLatitude = L.latitude_deg
            self.m_nTimezoneId = L.dst
            self.m_strTimeZone = GCTimeZone.GetTimeZoneName(id=self.m_nTimezoneId)
            self.m_fTimezone = GCTimeZone.GetTimeZoneOffset(id=self.m_nTimezoneId)
        elif isinstance(L,GCLocation):
            self.m_strTimeZone = L.m_strTimeZone
            self.m_strName = L.m_strName
            self.m_strCity = L.m_strCity
            self.m_strCountry = L.m_strCountry
            self.m_fLongitude = L.m_fLongitude
            self.m_fLatitude = L.m_fLatitude
            self.m_fTimezone = L.m_fTimezone
            self.m_nTimezoneId = L.m_nTimezoneId




class GCEvent:
    def __init__(self,data=None):
        if data:
            self.data = data
        else:
            self.data = {
                'cls': 0,
                'tithi': 0,
                'masa': 0,
                'fast': 0,
                'visible': 1,
                'used': 1,
                'spec': 0
            }

    @property
    def nTithi(self):
        return self.data['tithi']

    @nTithi.setter
    def nTithi(self,val):
        self.data['tithi'] = val

    @property
    def nMasa(self):
        return self.data['masa']

    @nMasa.setter
    def nMasa(self,val):
        self.data['masa'] = val

    @property
    def nClass(self):
        return self.data['cls']

    @nClass.setter
    def nClass(self,val):
        self.data['cls'] = val

    @property
    def nFastType(self):
        return self.data['fast']

    @nFastType.setter
    def nFastType(self,val):
        self.data['fast'] = val

    #---------------------
    @property
    def nVisible(self):
        return self.data['visible']

    @nVisible.setter
    def nVisible(self,val):
        self.data['visible'] = val

    #---------------------
    @property
    def nStartYear(self):
        return self.data['start']

    @nStartYear.setter
    def nStartYear(self,val):
        self.data['start'] = val

    #---------------------
    @property
    def nUsed(self):
        return self.data['used']

    @nUsed.setter
    def nUsed(self,val):
        self.data['used'] = val

    #---------------------
    @property
    def nSpec(self):
        return self.data['spec']

    @nSpec.setter
    def nSpec(self,val):
        self.data['spec'] = val

    #---------------------
    @property
    def strFastSubject(self):
        return self.data['fastSubj']

    @strFastSubject.setter
    def strFastSubject(self,val):
        self.data['fastSubj'] = val

    #---------------------
    @property
    def strText(self):
        return self.data['text']

    @strText.setter
    def strText(self,val):
        self.data['text'] = val



from gaurabda.GCEvent import GCEvent
import gaurabda.GCUT as GCUT
import os
import os.path
import json

list = []

def add():
    c = GCEvent()
    list.append(c)
    return c

def OpenFile(fileName):
    global list
    if not os.path.exists(fileName):
        fileName = os.path.join(os.path.dirname(__file__), 'res', 'events.json')
    with open(fileName,'rt',encoding='utf-8') as rf:
        events = json.load(rf)
        for e in events:
            list.append(GCEvent(data=e))
    return len(list)

def SaveFile(fileName):
    with open(fileName,'wt',encoding='utf-8') as wf:
        events = [ce.data for ce in list]
        wf.write(json.dumps(list,indent=4))
    return len(list)

def clear():
    list = []

def SetOldStyleFasting(bOldStyle):
    locMatrix = []
    file_name = os.path.join(os.path.dirname(__file__), 'res', 'eventfast.json')
    with open(file_name,'rt',encoding='utf-8') as rf:
        locMatrix = json.load(rf)
    ret = 0
    key = 'fast' if bOldStyle else 'newfast'
    for a in locMatrix:
        for pce in list:
            if pce.nMasa == a['masa'] and pce.nTithi == a['tithi'] and pce.nClass == a['cls']:
                if pce.nFastType != a[key]:
                    ret += 1
                    pce.nFastType = a[key]
                break
    return ret

def Count():
    return len(list)

def EventAtIndex(index):
    return list[index]


def unittests():
    GCUT.info('custom events')
    a = OpenFile('events.json')
    GCUT.nval(a,0,'open file')
    print(list[0].data)



from gaurabda.GCLocation import GCLocation
import gaurabda.GCCountry as GCCountry
import gaurabda.GCTimeZone as GCTimeZone
import os
import os.path
import json
import gaurabda.GCUT as GCUT

locationList = []
m_bModified = True

def OpenFile(pszFileList):
    if not os.path.exists(pszFileList):
        pszFileList = os.path.join(os.path.dirname(__file__), 'res', 'locations.json')
    ImportFile(pszFileList,True)

def IsModified():
    return m_bModified

def SaveAs(lpszFileName):
    locations = [a.data() for a in locationList]
    with open(lpszFileName,'wt',encoding='utf-8') as json_file:
        json_file.write(json.dumps(locations))

def ImportFile(pszFile, bDeleteCurrent=False):
    global locationList
    with open(pszFile,'rt',encoding='utf-8') as json_file:
        locations = json.load(json_file)
        if bDeleteCurrent:
            locationList=[]
        for a in locations:
            locationList.append(GCLocation(data=a))
        m_bModified = True

def RemoveAll():
    locationList=[]
    m_bModified=True

def Add(loc):
    locationList.append(loc)
    m_bModified = True

def RemoveAt(index):
    del locationList[index]
    m_bModified = True

def RenameCountry(pszOld, pszNew):
    for loc in locationList:
        if loc.m_strCountry==pszOld:
            loc.m_strCountry = pszNew
            m_pModified = True

def LocationCount():
    return len(locationList)

def LocationAtIndex(index):
    return locationList[index]

def unittests():
    GCUT.info('location list')
    OpenFile('locations.json')
    GCUT.nval(LocationCount(),0,'location count')
    lc = LocationAtIndex(0)
    print(lc.data())
